unit fEncounterFrame;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Tabs, ComCtrls, ExtCtrls, Menus, StdCtrls, Buttons, fPCEBase,
  fVisitType, fDiagnoses, fProcedure, fImmunization, fSkinTest, fPatientEd,
  fHealthFactor, fExam, uPCE, rPCE, rTIU, ORCtrls, ORFn, fBase508Form,
  VA508AccessibilityManager, uGN_Const, fRptBox, System.Actions, Vcl.ActnList;

const
  // tab names
  CT_VisitNm = '&Visit Type';
  CT_DiagNm = '&Diagnoses';
  CT_ProcNm = '&Procedures';
  CT_ImmNm = '&Immunizations';
  CT_SkinNm = '&Skin Tests';
  CT_PedNm = 'Patient &Ed';
  CT_HlthNm = '&Health Factors';
  CT_XamNm = 'E&xams';
  CT_VitNm = 'Vit&als';
  CT_GAFNm = '&GAF';

  NUM_TABS = 3;
  TAG_VTYPE = 10;
  TAG_DIAG = 20;
  TAG_PROC = 30;
  TAG_IMMUNIZ = 40;
  TAG_SKIN = 50;
  TAG_PED = 60;
  TAG_HF = 70;
  TAG_XAM = 80;
  TAG_TRT = 90;

  TX_NOSECTION = '-1^No sections found';
  TX_PROV_REQ =
    'A primary encounter provider must be selected before encounter data can' +
    CRLF + 'be saved.  Select the Primary Encounter Provider on the VISIT TYPE tab.'
    + CRLF + 'Otherwise, press <Cancel> to quit without saving data.';

  TC_PROV_REQ = 'Missing Primary Provider for Encounter';

type
  TfrmEncounterFrame = class(TfrmBase508Form)
    pnlPage: TPanel;
    Bevel1: TBevel;
    TabControl: TTabControl;
    mmLog: TMemo;
    alEncounterFrame: TActionList;
    acDebug: TAction;
    pnlToolbar: TPanel;
    acDebugProvider: TAction;
    popMenu: TPopupMenu;
    Debug1: TMenuItem;
    DebugProviderDlg1: TMenuItem;

    procedure tabPageChange(Sender: TObject; NewTab: Integer;
      var AllowChange: Boolean);
    procedure FormResize(Sender: TObject);
    procedure SectionClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure TabControlChange(Sender: TObject);
    procedure TabControlChanging(Sender: TObject; var AllowChange: Boolean);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCanResize(Sender: TObject; var NewWidth, NewHeight: Integer;
      var Resize: Boolean);
    procedure FormShow(Sender: TObject);
    procedure TabControlEnter(Sender: TObject);
    procedure acDebugExecute(Sender: TObject);
    procedure acDebugProviderExecute(Sender: TObject);

  private
{$IFDEF DEBUG}
    fDebugPCE: TPCEData;
{$ENDIF}
    FChangeSource: Integer;
    FCancel: Boolean;    // Indicates the cancel button has been pressed;
    FAbort: Boolean;     // Indicates that neither OK or Cancel has been pressed
    FormList: TStringList; // Holds the types of forms added to the frame
                         // They must be available at compile time
    FLastPage: TfrmPCEBase;
    FGiveMultiTabMessage: Boolean;
    procedure SynchPCEData;
    procedure SwitchToPage(NewForm: TfrmPCEBase); // was tfrmPage
    function FormListContains(item: string): Boolean;
    procedure FormListClear;
    procedure SendData(aGroupSave: Boolean = False);
    procedure UpdateEncounterByInput(PCE: TPCEData; FormRequest: Boolean = True);
    procedure doCreateForms;
    function pageByName(aName: String): TfrmPCEBase;
    procedure doAssignTabsVisibility(aMHClinic, aGroupEdit: Boolean);
  public
    property ChangeSource: Integer read FChangeSource;
    property Forms: TStringList read FormList;
    property Cancel: Boolean read FCancel write FCancel;
    property Abort: Boolean read FAbort write FAbort;
    procedure FormListCleanUp;
    procedure SetFormFonts;
  end;

var
  frmEncounterFrame: TfrmEncounterFrame;

  uVisitType: TPCEProc; // contains info for visit type page
  uEncPCEData: TPCEData;
  uProviders: TPCEProviderList;

function UpdatePCE(const PCEData: TPCEData; bGroup: Boolean): Boolean;

implementation

uses
  uCore, fGAF, uConst,
  rCore, fPCEProvider, rMisc, VA508AccessibilityRouter, VAUtils, fEncVitals,
  rGN_Core, oGN_PTData, uGN_Utils, uGN_RPCLog, rVitals;

{$R *.DFM}

// Returns TRUE if dialog was closed by OK button
//
// Note: Dialog data will be saved if OK Clicked
//       If GroupEdit is TRUE saving is done for all records
//
function UpdatePCE(const PCEData: TPCEData; bGroup: Boolean): Boolean;
var
  bMHClinic: Boolean;
  AUser: string;

begin
  if PCEData <> uEncPCEData then
    PCEData.CopyPCEData(uEncPCEData);
{$IFDEF DEBUG}
  addLogLine(PCEDataToString(PCEData), 'UpdatePCE: PCEData');
{$ENDIF}
  if (uEncPCEData.Empty and ((uEncPCEData.Location = 0) or
    (uEncPCEData.VisitDateTime = 0)) and (not Encounter.NeedVisit)) then
    uEncPCEData.UseEncounter := True;

  uProviders.Assign(uEncPCEData.Providers);
  SetDefaultProvider(uProviders, uEncPCEData);

  AUser := IntToStr(uProviders.PendingIEN(False));
  if (AUser <> '0') and (uProviders.IndexOfProvider(AUser) < 0) and
    AutoCheckout(uEncPCEData.Location) then
    uProviders.AddProvider(AUser, uProviders.PendingName(False), False);

  bMHClinic := MHClinic(uEncPCEData.Location);

  if not assigned(frmEncounterFrame) then
    begin
      frmEncounterFrame := TfrmEncounterFrame.Create(Application);
      frmEncounterFrame.doCreateForms;
    end;

  try
//    frmEncounterFrame.FormListCleanup; -- clean of Pt data if form is reused

    frmEncounterFrame.doAssignTabsVisibility(bMHClinic, GroupEdit);
    SetFormPosition(frmEncounterFrame);

    with frmEncounterFrame do
    begin
      Caption := 'Encounter Form for ' + uEncPCEData.PatientName + '  ' +
        ExternalName(uEncPCEData.Location, 44) + '  (' +
        FormatFMDateTime('mmm dd,yyyy@hh:nn', uEncPCEData.VisitDateTime) + ')';
      SetRPCEncLocation(uEncPCEData.Location);
      SynchPCEData;
      TabControl.Tabindex := 0;
      TabControlChange(TabControl);
      SetFormFonts;

      Result := ShowModal = mrOK;

      if Result then
        SendData(GroupEdit); // moved here from CanClose to honor Cancel
    end;
  finally
    // release form until we have proper tabs cleanup
    frmEncounterFrame.FormListClear;
    frmEncounterFrame.Free;
    frmEncounterFrame := nil;
  end;
end;

function TfrmEncounterFrame.FormListContains(item: string): Boolean;
begin
  Result := (FormList.IndexOf(item) <> -1) and
    (TabControl.Tabs.IndexOf(item) <> -1); // consider visibility of the tab
end;

procedure TfrmEncounterFrame.SwitchToPage(NewForm: TfrmPCEBase);
// was TfrmPage);
{ unmerge/merge menus, bring page to top of z-order, call form-specific OnDisplay code }
begin
  if (NewForm = nil) or (FLastPage = NewForm) then
    Exit;
  if Assigned(FLastPage) then
    FLastPage.Hide;
  FLastPage := NewForm;

  NewForm.DisplayPage; // this calls BringToFront for the form
//  NewForm.BringToFront; // that should be enough without call to DisplayPage

{$IFDEF GROUPNOTES}
  NewForm.Show;
{$ENDIF}
end;

function TfrmEncounterFrame.pageByName(aName: String): TfrmPCEBase;
var
  i: Integer;
begin
  result := nil;
  i := FormList.IndexOf(aName);
  if i >= 0 then
    result := TfrmPCEBase(FormList.Objects[i]);
end;

procedure TfrmEncounterFrame.tabPageChange(Sender: TObject; NewTab: Integer;
  var AllowChange: Boolean);
{ switches to form linked to NewTab }
begin
  SwitchToPage(pageByName(TabControl.Tabs[NewTab]));
end;

{ Resize and Font-Change procedures ------------------------------------------ }
procedure TfrmEncounterFrame.FormResize(Sender: TObject);
begin
  self.repaint;
end;

{ ///////////////////////////////////////////////////////////////////////////////
  //Name: TfrmEncounterFrame.SectionClick(Sender: TObject);
  //Created: Jan 1999
  //By: Robert Bott
  //Location: ISL
  //Description: Call the procedure apropriate for the selected tab
  /////////////////////////////////////////////////////////////////////////////// }
procedure TfrmEncounterFrame.SectionClick(Sender: TObject);
begin
  with Sender as TListBox do
    case Tag of
      TAG_VTYPE:
        if FormListContains(CT_VisitNm) then
        begin
          with frmVisitType do
            lstVTypeSectionClick(Sender);
        end;
    end;
end;

{ ///////////////////////////////////////////////////////////////////////////////
  //Name: procedure TfrmEncounterFrame.SynchPCEData;
  //Created: Jan 1999
  //By: Robert Bott
  //Location: ISL
  //Description: Synchronize any existing PCE data with what is displayed in the form.
  /////////////////////////////////////////////////////////////////////////////// }
procedure TfrmEncounterFrame.SynchPCEData;
var
  uSCCond: TSCConditions;

  procedure InitList(AListBox: TORListBox);
  var
    DoClick: Boolean;

  begin
    with AListBox do
    begin
      DoClick := True;
      case Tag of
        TAG_VTYPE:
          begin
            if FormListContains(CT_VisitNm) then
              ListVisitTypeSections(Items);
            DoClick := AutoSelectVisit(PCERPCEncLocation);
          end;
      end;
      if Items.Count > 0 then
      begin
        if DoClick then
        begin
          ItemIndex := 0;
          SectionClick(AListBox);
        end;
      end
      else
        Items.Add(TX_NOSECTION);
    end;
  end;

begin
{$IFDEF DEBUG}
    addLogLine(PCEDataToString(uEncPCEData), 'SynchPCEData - uEncPCEData');
{$ENDIF}
  if FormListContains(CT_VisitNm) then
    with frmVisitType do
    begin
      InitList(frmVisitType.lstVTypeSection); // set up Visit Type page
      if GroupEdit then
        begin
          memSCDisplay.Clear;
          memSCDisplay.Hint := 'No data shown for multile patients';
          memSCDisplay.ShowHint := True;
        end
      else
        begin
          ListSCDisabilities(memSCDisplay.Lines);
          memSCDisplay.ShowHint := False;
        end;
      uSCCond := EligbleConditions;
      frmVisitType.fraVisitRelated.InitAllow(uSCCond);
    end;

  with uEncPCEData do // load any existing data from PCEData
  begin
    if FormListContains(CT_VisitNm) then
      frmVisitType.fraVisitRelated.InitRelated(uEncPCEData);
    if FormListContains(CT_DiagNm) then
      frmDiagnoses.InitTab(CopyDiagnoses, ListDiagnosisSections);
    if FormListContains(CT_ProcNm) then
      frmProcedures.InitTab(CopyProcedures, ListProcedureSections);
    if FormListContains(CT_ImmNm) then
      frmImmunizations.InitTab(CopyImmunizations, ListImmunizSections);
    if FormListContains(CT_SkinNm) then
      frmSkinTests.InitTab(CopySkinTests, ListSkinSections);
    if FormListContains(CT_PedNm) then
      frmPatientEd.InitTab(CopyPatientEds, ListPatientSections);
    if FormListContains(CT_HlthNm) then
      frmHealthFactors.InitTab(CopyHealthFactors, ListHealthSections);
    if FormListContains(CT_XamNm) then
      frmExams.InitTab(CopyExams, ListExamsSections);
    uVisitType.Assign(VisitType);
    if FormListContains(CT_VisitNm) then
      frmVisitType.MatchVType;
  end;
end;

procedure TfrmEncounterFrame.FormListClear;
var
  i: Integer;
begin
  for i := ComponentCount - 1 downto 0 do
    if (Components[i] is TForm) then
      TForm(Components[i]).Free;

  FormList.clear;
end;

{ ///////////////////////////////////////////////////////////////////////////////
  //Name: procedure TfrmEncounterFrame.FormDestroy(Sender: TObject);
  //Created: Jan 1999
  //By: Robert Bott
  //Location: ISL
  //Description: Free up objects in memory when destroying form.
  /////////////////////////////////////////////////////////////////////////////// }
procedure TfrmEncounterFrame.FormDestroy(Sender: TObject);
begin
  inherited;
  FormListClear;
  FormList.Free;
{$IFDEF DEBUG}
  fDebugPCE.Free;
{$ENDIF}
end;

{ ///////////////////////////////////////////////////////////////////////////////
  //Name: procedure TfrmEncounterFrame.FormCreate(Sender: TObject);
  //Created: Jan 1999
  //By: Robert Bott
  //Location: ISL
  //Description: Create instances of the objects needed.
  /////////////////////////////////////////////////////////////////////////////// }
procedure TfrmEncounterFrame.FormCreate(Sender: TObject);
begin
  FormList := TStringList.Create;
  FCancel := false;
  FAbort := True;
  FGiveMultiTabMessage := ScreenReaderSystemActive;
{$IFDEF DEBUG}
  fDebugPCE := TPCEData.Create;
  TabControl.PopupMenu := popMenu;
{$ENDIF}
  SetUserBounds(TControl(self));
  adjustFormPosition(self);
end;

{ ///////////////////////////////////////////////////////////////////////////////
  //Name: procedure TfrmEncounterFrame.SendData;
  //Created: Jan 1999
  //By: Robert Bott
  //Location: ISL
  //Description: Send Data back to the M side sor storing.
  /////////////////////////////////////////////////////////////////////////////// }
procedure TfrmEncounterFrame.SendData(aGroupSave: Boolean = False);
// sends PCE data to the RPC
var
  StoreMessage, ptDFN: String;
  i: Integer;
  GAFScore: Integer;
  GAFDate: TFMDateTime;
  GAFStaff: Int64;
  PatientData: TPatientData;

  Procedure processGAF(aDFN: String);
  begin
    if (FormListContains(CT_GAFNm)) then
    begin
      frmGAF.GetGAFScore(GAFScore, GAFDate, GAFStaff);
      if (GAFScore > 0) then
        SaveGAFScore(GAFScore, GAFDate, GAFStaff);
    end;
  end;

  procedure processVitals;
  begin
    if FormListContains(CT_VitNm) then
    begin
      with frmEncVitals do
        if HasData then
        begin
          if AssignVitals then
          begin
            StoreMessage := ValAndStoreVitals(frmEncVitals.VitalNew);
            if (StoreMessage <> '') then
              InfoBox(StoreMessage, 'Saving Vitals',
                MB_OK or MB_ICONEXCLAMATION);
          end;
        end;
    end;
  end;

begin
  inherited;

  UpdateEncounterByInput(uEncPCEData); // updates Encounter with user input

{$IFDEF DEBUG}
  addLogLine(PCEDataToString(uEncPCEData), 'uEncPCEData - user input');
{$ENDIF}

  if aGroupSave then
  begin
    for i := 0 to GNPtList.Count - 1 do
    begin
      PatientData := TPatientData(GNPtList.Items[i]);
      ptDFN := PatientData.Patient.DFN;
{$IFDEF DEBUG}
      addLogLine(PCEDataToString(PatientData.PCEData), 'Patient.PCEData - before user input');
{$ENDIF}
      PatientData.ErasePCEData;
{$IFDEF DEBUG}
      addLogLine(PCEDataToString(uEncPCEData), 'uEncPCEData - before resetting flags');
{$ENDIF}
      uEncPCEData.setAppendFlags;
{$IFDEF DEBUG}
      addLogLine(PCEDataToString(uEncPCEData), 'uEncPCEData - after resetting flags');
{$ENDIF}
      uEncPCEData.CopyPCEData(PatientData.PCEData);
      PatientData.PCEData.Save(PatientData.PtNote.NoteIEN, ptDFN, True);
{$IFDEF DEBUG}
      addLogLine(PCEDataToString(PatientData.PCEData), 'Patient.PCEData - after user input');
{$ENDIF}
    end;
  end
  else
  begin
    processVitals;
    processGAF(Patient.DFN);
    PatientData := GetPatientData(Patient.DFN);
//    PatientData.ErasePCEData;
    // only one patient is saved - use TRUE to clean lists after save
//    uEncPCEData.setFlags(True,False);
    uEncPCEData.Save(uEncPCEData.NoteIEN, Patient.DFN, True);
    uEncPCEData.CopyPCEData(PatientData.PCEData);
  end;
end;

{ ///////////////////////////////////////////////////////////////////////////////
  //Name: procedure TfrmEncounterFrame.FormCloseQuery(Sender: TObject;
  //Created: Jan 1999
  //By: Robert Bott
  //Location: ISL
  //Description: Check to see if the Cancel button was pressed, if not, call
  // procedure to send the data to the server.
  /////////////////////////////////////////////////////////////////////////////// }
procedure TfrmEncounterFrame.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);

const
  TXT_SAVECHANGES = 'Save Changes?';

var
  sPatient: String;
  TmpPCEData: TPCEData;
  ask, ChangeOK: Boolean;

begin
  CanClose := True;
  if (FAbort) then
    FCancel := (InfoBox(TXT_SAVECHANGES, TXT_SAVECHANGES, MB_YESNO + MB_ICONQUESTION) = ID_NO);
  if FCancel then
    Exit; // *KCM*

  if (uProviders.PrimaryIdx >= 0) then
    ask := True
  else
  begin
    TmpPCEData := TPCEData.Create;
    try
      uEncPCEData.CopyPCEData(TmpPCEData);
      UpdateEncounterByInput(TmpPCEData);
      ask := TmpPCEData.NeedProviderInfo;
    finally
      TmpPCEData.Free;
    end;
  end;

  if GroupEdit then
    sPatient := 'Group of Patients'
  else
    sPatient := uEncPCEData.PatientName;

  if ask and (NoPrimaryPCEProvider(uProviders, uEncPCEData, sPatient)) then
  begin
    InfoBox(TX_PROV_REQ, TC_PROV_REQ, MB_OK or MB_ICONWARNING);
    CanClose := false;
    Exit;
  end;

  uVisitType.Provider := uProviders.PrimaryIEN; { RV - v20.1 }

  if FormListContains(CT_VitNm) then
  begin
    CanClose := frmEncVitals.OK2SaveVitals;
    if not CanClose then
    begin
      tabPageChange(self, TabControl.Tabs.IndexOf(CT_VitNm), ChangeOK);
      SwitchToPage(pageByName(CT_VitNm));
      TabControl.Tabindex := TabControl.Tabs.IndexOf(CT_VitNm);
      exit;
    end;
  end;

  if CanClose and FormListContains(CT_ProcNm) then
  begin
    CanClose := frmProcedures.OK2SaveProcedures;
    if not CanClose then
    begin
      tabPageChange(self, TabControl.Tabs.IndexOf(CT_ProcNm), ChangeOK);
      SwitchToPage(pageByName(CT_ProcNm));
      TabControl.Tabindex := TabControl.Tabs.IndexOf(CT_ProcNm);
    end;
  end;

  if CanClose then
    ModalResult := mrOK;
end;

procedure TfrmEncounterFrame.TabControlChange(Sender: TObject);
begin
  // must switch based on caption, as all tabs may not be present.
  if (Sender as TTabControl).Tabindex = -1 then
    Exit;

  SwitchToPage(pageByName(TabControl.Tabs[TabControl.Tabindex])); //<<<- CHeck if it is problem then  beware!!!


  if self.Visible and TabControl.CanFocus and Assigned(FLastPage) and not TabControl.Focused then
    TabControl.SetFocus; // CQ: 14845
end;

procedure TfrmEncounterFrame.TabControlChanging(Sender: TObject;
  var AllowChange: Boolean);
begin
  if (Assigned(FLastPage)) then
    FLastPage.AllowTabChange(AllowChange);
end;

procedure TfrmEncounterFrame.UpdateEncounterByInput(PCE: TPCEData;
  FormRequest: Boolean = True);
begin
  with PCE do
  begin
    if FormListContains(CT_VisitNm) then
    begin
      VisitType := uVisitType;
      frmVisitType.fraVisitRelated.GetRelated(uEncPCEData);
      Providers.Merge(uProviders);
    end;
    // ZZZZZZBELLC
    if FormListContains(CT_DiagNm) then
      SetDiagnoses(frmDiagnoses.lstCaptionList.ItemsStrings, FormRequest);
    if FormListContains(CT_ProcNm) then
      SetProcedures(frmProcedures.lstCaptionList.ItemsStrings, FormRequest);
    if FormListContains(CT_ImmNm) then
      SetImmunizations(frmImmunizations.lstCaptionList.ItemsStrings,
        FormRequest);
    if FormListContains(CT_SkinNm) then
      SetSkinTests(frmSkinTests.lstCaptionList.ItemsStrings, FormRequest);
    if FormListContains(CT_PedNm) then
      SetPatientEds(frmPatientEd.lstCaptionList.ItemsStrings, FormRequest);
    if FormListContains(CT_HlthNm) then
      SetHealthFactors(frmHealthFactors.lstCaptionList.ItemsStrings,
        FormRequest);
    if FormListContains(CT_XamNm) then
      SetExams(frmExams.lstCaptionList.ItemsStrings, FormRequest);
  end;
end;
{
procedure TfrmEncounterFrame.SelectTab(NewTabName: string);
var
  AllowChange: Boolean;
begin
  AllowChange := True;
//  TabControl.Tabindex := FormList.IndexOf(NewTabName);  <--- BAD!
  TabControl.Tabindex := TabControl.Tabs.IndexOf(NewTabName);
  tabPageChange(self, TabControl.Tabindex, AllowChange);
end;
}
procedure TfrmEncounterFrame.TabControlEnter(Sender: TObject);
begin
  if FGiveMultiTabMessage then // CQ#15483
  begin
    FGiveMultiTabMessage := false;
    GetScreenReader.Speak('Multi tab form');
  end;
end;

procedure TfrmEncounterFrame.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
  CanChange: Boolean;
begin
  inherited;
  if (Key = VK_ESCAPE) then
  begin
    Key := 0;
    FLastPage.btnCancel.Click;
  end
  else if Key = VK_TAB then
  begin
    if ssCtrl in Shift then
    begin
      CanChange := True;
      if Assigned(TabControl.OnChanging) then
        TabControl.OnChanging(TabControl, CanChange);
      if CanChange then
      begin
        if ssShift in Shift then
        begin
          if TabControl.Tabindex < 1 then
            TabControl.Tabindex := TabControl.Tabs.Count - 1
          else
            TabControl.Tabindex := TabControl.Tabindex - 1;
        end
        else
          TabControl.Tabindex := (TabControl.Tabindex + 1)
            mod TabControl.Tabs.Count;
        if Assigned(TabControl.OnChange) then
          TabControl.OnChange(TabControl);
      end;
      Key := 0;
    end;
  end;
end;

procedure TfrmEncounterFrame.SetFormFonts;
var
  i: Integer;
  NewFontSize: Integer;
begin
  NewFontSize := MainFontsize;
  Font.Size := NewFontSize;
  for i := 0 to FormList.Count - 1 do
    if Assigned(FormList.Objects[i]) and (FormList.Objects[i] is TfrmPCEBase)
    then
      TfrmPCEBase(FormList.Objects[i]).setFontSize(NewFontSize);
  if Assigned(frmVisitType) then
    frmVisitType.setFontSize(NewFontSize); // sets fraVisitRelated font size

  pnlToolbar.Height := getMainFormTextHeight + GAP + GAP;

  GN_ResizeAnchoredFormToFont(self);
end;

procedure TfrmEncounterFrame.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  SaveUserBounds(self);
end;

procedure TfrmEncounterFrame.FormCanResize(Sender: TObject;
  var NewWidth, NewHeight: Integer; var Resize: Boolean);
begin
  // CQ4740
  if NewWidth < 200 then
  begin
    NewWidth := 200;
    Resize := false;
  end;
end;

procedure TfrmEncounterFrame.FormShow(Sender: TObject);
begin
  inherited;
  if TabControl.CanFocus and Assigned(FLastPage) and not TabControl.Focused then
    TabControl.SetFocus;
end;

procedure TfrmEncounterFrame.acDebugExecute(Sender: TObject);
{$IFDEF DEBUG}
  procedure UpdateDebugInfo;
  var
    i: Integer;
    sl: TStringList;
    s, ss: String;
  begin;
    UpdateEncounterByInput(fDebugPCE, false);
    mmLog.clear;
    mmLog.Lines.Add(FormatDateTime('-- mm/dd/yyyy hh:nn:ss --', Now));
    mmLog.Lines.Add('-------------------------');
    mmLog.Lines.Text := mmLog.Lines.Text + CRLF + PCEDataToString(fDebugPCE);
    i := 1;
    while i <= mmLog.Lines.Count - 1 do
      if Trim(mmLog.Lines[i]) = '' then
        mmLog.Lines.Delete(i)
      else
        inc(i);
    mmLog.Lines.Add('-------------------------');
    sl := fDebugPCE.getPCEDataList;
    for s in sl do
    begin
      ss := StringReplace(s, #0, '#0', [rfReplaceAll]);
      mmLog.Lines.Add(ss);
    end;
    sl.Free;
  end;

begin
  inherited;
  UpdateDebugInfo;
  ReportBox(mmLog.Lines, 'Encounter Debug Report', false);
{$ELSE}
begin
{$ENDIF}
end;

procedure TfrmEncounterFrame.acDebugProviderExecute(Sender: TObject);
begin
  inherited;
  if (NoPrimaryPCEProvider(uProviders, uEncPCEData)) then
    InfoBox(TX_PROV_REQ, TC_PROV_REQ, MB_OK or MB_ICONWARNING);
end;

procedure TfrmEncounterFrame.doCreateForms;
var
  j: Integer;
begin
  FormListClear;

  frmVisitType := TfrmVisitType.CreateLinked(pnlPage);
  frmDiagnoses := TfrmDiagnoses.CreateLinked(pnlPage);
  frmProcedures := TfrmProcedures.CreateLinked(pnlPage);
  frmEncVitals := TfrmEncVitals.CreateLinked(pnlPage);
  frmImmunizations := TfrmImmunizations.CreateLinked(pnlPage);
  frmSkinTests := TfrmSkinTests.CreateLinked(pnlPage);
  frmPatientEd := TfrmPatientEd.CreateLinked(pnlPage);
  frmHealthFactors := TfrmHEalthFactors.CreateLinked(pnlPage);
  frmExams := TfrmExams.CreateLinked(pnlPage);
  frmGAF := TfrmGAF.CreateLinked(pnlPage);

  FormList.AddObject(CT_VisitNm, frmVisitType);
  FormList.AddObject(CT_DiagNm, frmDiagnoses);
  FormList.AddObject(CT_ProcNm, frmProcedures);
  FormList.AddObject(CT_VitNm, frmEncVitals);
  FormList.AddObject(CT_ImmNm, frmImmunizations);
  FormList.AddObject(CT_SkinNm, frmSkinTests);
  FormList.AddObject(CT_PedNm, frmPatientEd);
  FormList.AddObject(CT_HlthNm, frmHealthFactors);
  FormList.AddObject(CT_XamNm, frmExams);
  FormList.AddObject(CT_GAFNm, frmGAF);

  frmVisitType.Visible := True;
  for j := 1 to FormList.Count - 1 do
    TForm(FormList.Objects[j]).Visible := false;

end;

procedure TfrmEncounterFrame.doAssignTabsVisibility(aMHClinic, aGroupEdit: Boolean);
var
  ss,
  s: String;

  function CheckName(aName:String): Boolean;
  begin
    if aGroupEdit then
      Result := (pos(aName,ss)=0)
    else
      if (aName = CT_GAFNm) then
        Result := aMHClinic
      else
        Result := True;
  end;

begin
  TabControl.Tabs.Clear;
  ss := CT_VitNm + U + CT_SkinNm + U + CT_XamNm + U + CT_GAFNm;
  for s in FormList do
    begin
      if CheckName(s) then
        TabControl.Tabs.Add(s);
    end;
end;

procedure TfrmEncounterFrame.FormListCleanup;
var
  i: Integer;
begin
  for i := ComponentCount - 1 downto 0 do
    if Components[i] is TfrmPCEBase then
      TfrmPCEBase(Components[i]).CleanUp;
end;

initialization

uEncPCEData := TPCEData.Create;
uProviders := TPCEProviderList.Create;
uVisitType := TPCEProc.Create;

finalization

uEncPCEData.Free;
uProviders.Free;
uVisitType.Free;

end.
