unit fGAF;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  fPCEBase, StdCtrls, Buttons, ExtCtrls, Grids, ORFn, ORNet, ORCtrls,
  ORDtTm, ComCtrls, fPCEBaseGrid, Menus, VA508AccessibilityManager,
  u508Extensions;

type
  TfrmGAF = class(TfrmPCEBaseGrid)
    lblGAF: TStaticText;
    btnURL: u508Extensions.TButton;
    gpMain: TGridPanel;
    lblScore: TLabel;
    lblDate: TLabel;
    dteGAF: TORDateBox;
    lblDeterminedBy: TLabel;
    cboGAFProvider: TORComboBox;
    splGAF: TSplitter;
    pnlScore: TPanel;
    edtScore: TCaptionEdit;
    udScore: TUpDown;
    lblEntry: TStaticText;
    procedure cboGAFProviderNeedData(Sender: TObject; const StartFrom: String;
      Direction, InsertAt: Integer);
    procedure edtScoreChange(Sender: TObject);
    procedure cboGAFProviderExit(Sender: TObject);
    procedure btnURLClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Init;
    procedure FormShow(Sender: TObject);
    procedure udScoreEnter(Sender: TObject);
    procedure udScoreExit(Sender: TObject);
    procedure edtScoreEnter(Sender: TObject);
    procedure edtScoreExit(Sender: TObject);
    procedure dteGAFExit(Sender: TObject);
    procedure dteGAFEnter(Sender: TObject);
    procedure cboGAFProviderEnter(Sender: TObject);
  private
    procedure LoadScores;
    function BADData(ShowMessage: boolean): boolean;
  public
    procedure AllowTabChange(var AllowChange: boolean); override;
    procedure GetGAFScore(var Score: Integer; var Date: TFMDateTime;
      var Staff: Int64);
    procedure CleanUp; override;
  end;

var
  frmGAF: TfrmGAF;

implementation

uses rPCE, rCore, uCore, uPCE, fEncounterFrame, VA508AccessibilityRouter,
  uGN_Utils, uGN_Const;

{$R *.DFM}

function ValidGAFData(Score: Integer; Date: TFMDateTime; Staff: Int64): boolean;
begin
  if (Score < 1) or (Score > 100) or (Date <= 0) or (Staff = 0) then
    Result := FALSE
  else
    Result := ((Patient.DateDied <= 0) or (Date <= Patient.DateDied));
end;

procedure TfrmGAF.LoadScores;
var
  i: Integer;
  tmp: string;

begin
  RecentGafScores(3);
  if (RPCBrokerV.Results.Count > 0) and (RPCBrokerV.Results[0] = '[DATA]') then
  begin
    for i := 1 to RPCBrokerV.Results.Count - 1 do
    begin
      tmp := RPCBrokerV.Results[i];
      lstCaptionList.Add(Piece(tmp, U, 5) + U + Piece(Piece(tmp, U, 2),
        NoPCEValue, 1) + U + Piece(tmp, U, 7) + U + Piece(tmp, U, 8));

    end;
  end;
  if lstCaptionList.Items.Count = 0 then
  begin
    lstCaptionList.Add('No GAF scores found.');
    lstCaptionList.ShowColumnHeaders := FALSE;
    lstCaptionList.Columns[0].Width :=
      getMainFormTextWidth('No GAF scores found.') + GAP + GAP;
  end
  else
  begin
    lstCaptionList.ShowColumnHeaders := True;
    lstCaptionList.Columns[0].Width := 120; // Original
  end;
end;

procedure TfrmGAF.udScoreEnter(Sender: TObject);
begin
  inherited;
{$IFDEF DEBUG}
  edtScore.Color := clInfoBk;
{$ENDIF}
end;

procedure TfrmGAF.udScoreExit(Sender: TObject);
begin
  inherited;
{$IFDEF DEBUG}
  edtScore.Color := clWindow;
{$ENDIF}
end;

procedure TfrmGAF.cboGAFProviderNeedData(Sender: TObject;
  const StartFrom: String; Direction, InsertAt: Integer);
begin
  inherited;
  cboGAFProvider.ForDataUse(SubSetOfPersons(StartFrom, Direction));
end;

procedure TfrmGAF.dteGAFEnter(Sender: TObject);
begin
  inherited;
{$IFDEF DEBUG}
  dteGAF.Color := clCream;
{$ENDIF}

end;

procedure TfrmGAF.dteGAFExit(Sender: TObject);
begin
  inherited;
{$IFDEF DEBUG}
  dteGAF.Color := clWindow;
{$ENDIF}
end;

function TfrmGAF.BADData(ShowMessage: boolean): boolean;
var
  PName, msg: string;
  GAFDate: TFMDateTime;
  UIEN: Int64;

begin
  GAFDate := dteGAF.FMDateTime;
  msg := ValidateGAFDate(GAFDate);
  if (dteGAF.FMDateTime <> GAFDate) then
    dteGAF.FMDateTime := GAFDate;

  if (cboGAFProvider.ItemID = '') then
  begin
    if (msg <> '') then
      msg := msg + CRLF;
    msg := msg + 'A determining party is required to enter a GAF score.';
    UIEN := uProviders.PCEProvider;
    if (UIEN <> 0) then
    begin
      PName := uProviders.PCEProviderName;
      msg := msg + '  Determined By changed to ' + PName + '.';
      cboGAFProvider.SelectByIEN(UIEN);
      if (cboGAFProvider.ItemID = '') then
      begin
        cboGAFProvider.InitLongList(PName);
        cboGAFProvider.SelectByIEN(UIEN);
      end;
    end;
  end;

  if (ShowMessage and (msg <> '')) then
    InfoBox(msg, 'Invalid GAF Data', MB_OK);

  if (udScore.Position > udScore.Min) then
    Result := (msg <> '')
  else
    Result := FALSE;
end;

procedure TfrmGAF.edtScoreChange(Sender: TObject);
var
  i: Integer;

begin
  inherited;
  i := StrToIntDef(edtScore.Text, udScore.Min);
  if (i < udScore.Min) or (i > udScore.Max) then
    i := udScore.Min;
  udScore.Position := i;
  edtScore.Text := IntToStr(i);
  edtScore.SelStart := length(edtScore.Text);
end;

procedure TfrmGAF.edtScoreEnter(Sender: TObject);
begin
  inherited;
{$IFDEF DEBUG}
  edtScore.Color := clCream;
{$ENDIF}

end;

procedure TfrmGAF.edtScoreExit(Sender: TObject);
begin
  inherited;
{$IFDEF DEBUG}
  edtScore.Color := clWindow;
{$ENDIF}

end;

procedure TfrmGAF.cboGAFProviderEnter(Sender: TObject);
begin
  inherited;
{$IFDEF DEBUG}
  cboGAFProvider.Color := clCream;
{$ENDIF}
end;

procedure TfrmGAF.cboGAFProviderExit(Sender: TObject);
begin
  inherited;
{$IFDEF DEBUG}
  cboGAFProvider.Color := clWindow;
{$ENDIF}
  BADData(True);
end;

procedure TfrmGAF.AllowTabChange(var AllowChange: boolean);
begin
  AllowChange := (not BADData(True));
end;

procedure TfrmGAF.GetGAFScore(var Score: Integer; var Date: TFMDateTime;
  var Staff: Int64);
begin
  Score := udScore.Position;
  if (Score > 0) then
    BADData(True);
  Date := dteGAF.FMDateTime;
  Staff := cboGAFProvider.ItemIEN;
  if (not ValidGAFData(Score, Date, Staff)) then
  begin
    Score := 0;
    Date := 0;
    Staff := 0
  end;
end;

procedure TfrmGAF.btnURLClick(Sender: TObject);
begin
  inherited;
  GotoWebPage(GAFURL);
end;

procedure TfrmGAF.FormCreate(Sender: TObject);
begin
  inherited;
  FTabName := CT_GAFNm;
  btnURL.Visible := (User.WebAccess and (GAFURL <> ''));
  Init;
end;

procedure TfrmGAF.FormShow(Sender: TObject);
begin
  inherited;
  udScore.Left := tPanel(udScore.Parent).Width - udScore.Width;
end;

procedure TfrmGAF.Init;
var
  i: Integer;
begin
  LoadScores;
  cboGAFProvider.InitLongList(Encounter.ProviderName);
  BADData(FALSE);

  i := getMainFormTextWidth(lblDeterminedBy.Caption) + 16;
  gpMain.ColumnCollection[1].Value := i;

//  i := getMainFormTextWidth(btnURL.Caption) + 16;
//  btnURL.Width := i;
//  i := getMainFormTextWidth(btnCancel.Caption) + 18;
//  btnCancel.Width := i;
//  btnOK.Width := i;
  adjustBtn(btnUrl);
  adjustBitBtn(btnOK);
  adjustBitBtn(btnCancel);

  gpMain.ColumnCollection[1].Value := btnUrl.Width - btnUrl.Margins.Left -
    gpMain.ColumnCollection[0].Value;
  gpMain.ColumnCollection[3].Value := { i + } btnCancel.Margins.Right;

  i := getMainFormTextHeight + 4;
  gpMain.RowCollection[0].Value := i;
  gpMain.RowCollection[1].Value := i;
end;

procedure TfrmGAF.CleanUp;
begin
  lstCaptionList.Clear;
  edtScore.Text := '';
  dteGAF.Clear;
  cboGAFProvider.Clear;
end;

initialization

SpecifyFormIsNotADialog(TfrmGAF);

end.
