unit fGN_About;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, VA508AccessibilityManager, fBase508Form, Vcl.AppEvnts
  ,u508Extensions;

type
  TfrmAbout = class(TfrmBase508Form)
    pnlCanvas: TPanel;
    Image1: TImage;
    cmdOK: u508Extensions.TButton;
    pnlButton: TPanel;
    pnlBackground: TPanel;
    ae: TApplicationEvents;
    gpMain: TGridPanel;
    lblCompanyName: TStaticText;
    lblProductName: TStaticText;
    lblComments: TStaticText;
    lblInternalName: TStaticText;
    lblOriginalFileName: TStaticText;
    lblFileDescription: TStaticText;
    lblCRC: TStaticText;
    lblLegalCopyright: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure aeIdle(Sender: TObject; var Done: Boolean);
    procedure cmdOKClick(Sender: TObject);
  private
    { Private declarations }
    fSplashHeight: Integer;
    fMode: String;
    procedure setMode(aMode: String);
  public
    { Public declarations }
    property Mode: String read fMode write setMode;
  end;

procedure ShowAbout;
procedure SplashAbout;
procedure SplashHide;

implementation

{$R *.DFM}

uses
  ORSystem, ORFn, VAUtils // ICD-10 Remediation
    , uGN_Utils;

var
  frmAbout: TfrmAbout;
  s508Warning: String =
    'Department of Veterans Affairs staff have made every effort ' +
    'during the design, development and testing of this application ' +

    'to ensure full accessibility to all users in compliance with ' +
    'Section 508 of the Rehabilitation Act of 1973, as amended.  ' +

    'If any issues are encountered while running this application, ' +
    'please contact your local help desk for support. ';

procedure SplashHide;
begin
  if assigned(frmAbout) then
    begin
      frmAbout.Hide;
      frmAbout.Free;
      frmAbout := nil;
    end;
  Application.ProcessMessages;
end;

procedure SplashAbout;

begin
  if not assigned(frmAbout) then
    frmAbout := TfrmAbout.Create(Application); // show splash screen
  with frmAbout do
  begin
    Mode := 'SPLASH';
    Show;
    Refresh;
  end;
end;

procedure ShowAbout;
begin
  if not assigned(frmAbout) then
    frmAbout := TfrmAbout.Create(Application);
  try
    with frmAbout do
    begin
      Mode := 'ABOUT';
      ShowModal;
    end;
  finally
    frmAbout.Free;
    frmAbout := nil;
  end;
end;

procedure TfrmAbout.aeIdle(Sender: TObject; var Done: Boolean);
begin
  inherited;
  // ae.OnIdle := nil; //does not work if is hidden only once :(
  HideCaret(lblLegalCopyright.Handle);
end;

procedure TfrmAbout.cmdOKClick(Sender: TObject);
begin
  inherited;
  Close;
end;

procedure TfrmAbout.FormCreate(Sender: TObject);
begin
  inherited;
  lblCompanyName.Caption := { 'Developed by the ' + } FileVersionValue
    (Application.ExeName, FILE_VER_COMPANYNAME);
  lblFileDescription.Caption := { 'Compiled: ' + } FileVersionValue
    (Application.ExeName, FILE_VER_FILEDESCRIPTION); // date
  // lblFileVersion.Caption        := FileVersionValue(Application.ExeName, FILE_VER_FILEVERSION);
  lblInternalName.Caption := FileVersionValue(Application.ExeName,
    FILE_VER_INTERNALNAME);
  lblLegalCopyright.Text := FileVersionValue(Application.ExeName,
    FILE_VER_LEGALCOPYRIGHT);
  lblOriginalFileName.Caption := FileVersionValue(Application.ExeName,
    FILE_VER_ORIGINALFILENAME); // patch
  lblProductName.Caption := FileVersionValue(Application.ExeName,
    FILE_VER_PRODUCTNAME);
  lblComments.Caption := FileVersionValue(Application.ExeName,
    FILE_VER_COMMENTS); // version comment
  lblCRC.Caption := 'CRC: ' + IntToHex(CRCForFile(Application.ExeName), 8);
{$IFDEF DEBUG}
  pnlButton.ShowCaption := True;
{$ENDIF}
  lblLegalCopyright.Text := lblLegalCopyright.Text + #13#10#13#10 + s508warning;
  lblLegalCopyright.SelStart := Length(lblLegalCopyright.Text);
  lblLegalCopyright.SelLength := 0;

  HideCaret(lblLegalCopyright.Handle);
  fSplashHeight := 0;
end;

procedure TfrmAbout.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  inherited;
  if Key = VK_ESCAPE then
    Close;
end;

procedure TfrmAbout.setMode(aMode: string);
begin
  if aMode <> fMode then
  begin
    fMode := aMode;
    if fMode = 'SPLASH' then
    begin
      Height := Height - pnlButton.Height - getTitleHeight(self);
//      lblLegalCopyright.Margins.Top := 20;
//      lblLegalCopyright.Margins.Bottom := 20;
      BorderStyle := bsNone;
      fSplashHeight := Height;
    end
    else
    begin
//      Height := Height + pnlButton.Height + getTitleHeight(self);
      BorderStyle := bsSingle;
//      lblLegalCopyright.Margins.Top := 8;
//      lblLegalCopyright.Margins.Bottom := 8;
      if fSplashHeight > 0 then
        Height := fSplashHeight + pnlButton.Height + getTitleHeight(self);
    end;
    pnlButton.Visible := fMode <> 'SPLASH';
  end;
end;

end.
