unit fGN_Debug;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants,
  System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, fBase508Form, VA508AccessibilityManager,
  Vcl.StdCtrls, Vcl.ComCtrls, Vcl.ExtCtrls, System.Actions,
  Vcl.ActnList, Vcl.Buttons, Vcl.ToolWin, System.ImageList, Vcl.ImgList,
  Vcl.Menus;

type
  TfrmDebug = class(TfrmBase508Form)
    pnlCanvas: TPanel;
    splDebug: TSplitter;
    tv: TTreeView;
    pnlTools: TPanel;
    sb: TStatusBar;
    pnlLeft: TPanel;
    Panel1: TPanel;
    pnlDetails: TPanel;
    edTarget: TEdit;
    alRTTI: TActionList;
    acAllTypes: TAction;
    mmDetails: TMemo;
    lblItem: TLabel;
    Label1: TLabel;
    ToolBar1: TToolBar;
    ToolButton10: TToolButton;
    ToolButton11: TToolButton;
    ToolButton12: TToolButton;
    ToolButton13: TToolButton;
    c: TImageList;
    acAlignLeft: TAction;
    acAlignRight: TAction;
    Panel2: TPanel;
    lvDetails: TListView;
    ToolButton1: TToolButton;
    acView: TAction;
    tmRTTI: TTimer;
    acAutoRefresh: TAction;
    pnlToolbar: TPanel;
    ToolBar2: TToolBar;
    ToolButton7: TToolButton;
    ToolButton8: TToolButton;
    ToolButton14: TToolButton;
    ToolButton2: TToolButton;
    mnuDebug: TMainMenu;
    File1: TMenuItem;
    acClose: TAction;
    Close1: TMenuItem;
    procedure tvChange(Sender: TObject; Node: TTreeNode);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure acAlignLeftExecute(Sender: TObject);
    procedure acAlignRightExecute(Sender: TObject);
    procedure acAllTypesExecute(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure acViewExecute(Sender: TObject);
    procedure tvDblClick(Sender: TObject);
    procedure tmRTTITimer(Sender: TObject);
    procedure acAutoRefreshExecute(Sender: TObject);
    procedure acCloseExecute(Sender: TObject);
  private
    { Private declarations }
    indent, indentStep: String;
    _Ignore: Boolean;
    ndTypes: TTreeNode;
    procedure LogClear;
    procedure Clear;
    procedure doTVChange(Node: TTreeNode);
    function findNodeObject(aNode: TTreeNode; anObj: TObject): Boolean;
//    procedure AddObjectList(aList: TStringList; aName: String;
//      addNils: Booean = False);
    procedure doGetTypes;

    procedure LogUpdateBegin;
    procedure LogUpdateEnd;

    procedure LogUpdate(aText: String); overload;
    procedure LogUpdate(aType, aName, aValue: String); overload;

    procedure WriteObjectRTTI(Node: TTreeNode);
    procedure RefreshNode(aNode: TTreeNode);
  public
    procedure AddObject(anObject: TObject; aName: String = '');
  end;

procedure ShowRTTI(anObject: TObject);
procedure CreateRTTI;

var
  oList: TList;
  frmDebug: TfrmDebug;

implementation

uses
  System.RTTI, uCore, uPCE, fEncounterFrame, oGN_PtData, ORNet, uGN_Utils,
  rCore
  ;

const
  CRLF = #13#10;

{$R *.dfm}


procedure setOList;

  procedure Add(anObject: TObject);
  begin
    if oList.IndexOf(anObject) < 0 then
      oList.Add(anObject);
  end;

begin
  Add(RPCBrokerV);
  Add(User);
  Add(uProviders);
  Add(RemoteSites);
  Add(RemoteReports);
  Add(Notifications);
  Add(Changes);
  Add(Encounter);
  Add(SavedEncounter);
  Add(uEncPCEData);
  Add(Patient);
  Add(GNPtList);
//  Add(BasePCE);
end;

function getObjectFieldByName(anObject: TObject; aName: String): String;
var
  aContext: TRttiContext;
  aType: TRttiType;
  aField: TRttiField;
  aVal: TValue;
begin
  try
    aType := aContext.GetType(anObject.ClassType);
    aField := aType.GetField(aName);
    if assigned(aField) then
      begin
        aVal := aField.GetValue(anObject);
        Result := aVal.ToString;
      end
    else
      Result := '';
  finally
  end;
end;

function getObjectPropertyByName(anObject: TObject; aName: String): String;
var
  aContext: TRttiContext;
  aType: TRttiType;
  aProperty: TRttiProperty;
  aVal: TValue;
begin
  try
    aType := aContext.GetType(anObject.ClassType);
    aProperty := aType.GetProperty(aName);
    if assigned(aProperty) then
      begin
        aVal := aProperty.GetValue(anObject);
        Result := aVal.ToString;
      end
    else
      Result := '';
  finally
  end;
end;

procedure CreateRTTI;
begin
  if not assigned(frmDebug) then
    Application.CreateForm(TfrmDebug, frmDebug);
end;

procedure ShowRTTI(anObject: TObject);
var
  s: String;
  obj: TObject;
  i: Integer;
begin
  if not assigned(frmDebug) then
    CreateRTTI;
  if assigned(frmDebug) then
  begin
    setOList;

    for I := 0 to oList.Count - 1 do
    begin
      obj := TObject(oList[i]);
      if assigned(obj) then
      begin
        s := getObjectFieldByName(obj, 'Name');
        frmDebug.AddObject(obj, s + ' (' + obj.ClassName + ', ' +
          IntToStr(Int64(obj)) + ')');
      end;
    end;
    if assigned(anObject) then
      frmDebug.AddObject(anObject);
    frmDebug.Show;
  end;
end;

/// /////////////////////////////////////////////////////////////////////////////

procedure TfrmDebug.tmRTTITimer(Sender: TObject);
begin
  inherited;
  if assigned(tv.Selected) then
    RefreshNode(tv.Selected);
end;

procedure TfrmDebug.tvChange(Sender: TObject; Node: TTreeNode);
begin
  if _Ignore then
    exit;
  doTVChange(Node);
end;

procedure TfrmDebug.tvDblClick(Sender: TObject);
begin
  inherited;
  if assigned(tv.Selected) then
    doTVChange(tv.Selected);
end;

function TfrmDebug.findNodeObject(aNode: TTreeNode; anObj: TObject): Boolean;
var
  nn: TTreeNode;
begin
  Result := False;
  nn := aNode.Parent;
  if assigned(nn) and (nn.Data = Pointer(anObj)) then
  begin
    Result := True;
    exit;
  end;

  nn := aNode.getFirstChild;
  while nn <> nil do
  begin
    Result := nn.Data = Pointer(anObj);
    if Result then
      exit
    else
      nn := nn.GetNext;
  end;
end;

procedure TfrmDebug.WriteObjectRTTI(Node: TTreeNode);
var
  p: Pointer;
  sName, aLine, aClassName: String;
  anObj, _Obj: TObject;
  aContext: TRttiContext;
  aType: TRttiType;
  aField: TRttiField;
  sl: TStrings;
  i: Integer;

const
  fmtNameValue = '%-25.25s %s';
  fmtNameClassValue = '%-25.25s (%-25.25s) %s';

  procedure addObjectToNode(anObject: TObject; aNode: TTreeNode);
  var
    _Name, _ClassName: String;
  begin
    if anObject = nil then
      exit;
    if findNodeObject(aNode, TObject(anObject)) then
      exit;

    _ClassName := TObject(anObject).ClassName;
    _Name := getObjectFieldByName(TObject(anObject), 'Name');
    tv.Items.AddChildObject(aNode, _Name + ' (' + _ClassName + ')',
      TObject(anObject));
  end;

begin
  anObj := TObject(Node.Data);
  LogClear;

  if anObj = nil then
    exit;
  if anObj.ClassType = nil then
    exit;

  LogUpdateBegin;

  if anObj is TStrings then
  begin
    sl := TStrings(anObj);
    for i := 0 to sl.Count - 1 do
    begin
      aLine := sl[i];
      _Obj := sl.Objects[i];
      if assigned(_Obj) then
      begin
        LogUpdate('', aLine, _Obj.ClassName);
        addObjectToNode(_Obj,Node);
      end;
    end;
  end
  else if anObj is TList then
  begin
    for p in TPointerList(TList(anObj).List) do
      addObjectToNode(p,Node);
  end
  else
    try
      aType := aContext.GetType(anObj.ClassType);
      for aField in aType.GetFields do
        if aField.FieldType.IsInstance then
        begin
          _Obj := nil;
          try
            _Obj := aField.GetValue(anObj).AsObject;
            if assigned(_Obj) then
            begin
              sName := getObjectFieldByName(_Obj, 'Name');
              aClassName := sName + ' (' + _Obj.ClassName + ')';
              if not findNodeObject(Node, _Obj) then
                tv.Items.AddChildObject(Node, aField.Name + aClassName, _Obj);
            end
            else
              aClassName := '';

            LogUpdate(indent + Format(fmtNameValue, [aField.Name, aClassName]));
            LogUpdate('Field', aField.Name, aClassName);
          except
            on E: Exception do
              if assigned(_Obj) then
                LogUpdate(_Obj.ClassName + CRLF + E.Message);
          end;
        end
        else
        begin
          LogUpdate(indent + Format(fmtNameValue,
            [aField.Name, aField.GetValue(anObj).ToString]));
          LogUpdate('Field', aField.Name, aField.GetValue(anObj).ToString);
        end;
    except
      on E: Exception do
      begin
        LogUpdate('ClassName <' + anObj.ClassType.ClassName + '>');
        LogUpdate(E.Message);
      end;
    end;

  LogUpdateEnd;
end;

procedure TfrmDebug.doTVChange(Node: TTreeNode);
begin
  inherited;
  LogClear;
  lblItem.Caption := Node.Text;
  WriteObjectRTTI(Node);
end;

procedure TfrmDebug.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  inherited;
  Hide;
  Action := caNone;
end;

procedure TfrmDebug.FormCreate(Sender: TObject);
begin
  inherited;
  indent := '';
  indentStep := '  ';
end;

procedure TfrmDebug.FormDestroy(Sender: TObject);
begin
  Clear;
  inherited;
end;

procedure TfrmDebug.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  inherited;
  if Key = VK_ESCAPE then
  begin
    Key := 0;
    Hide;
  end;
end;

procedure TfrmDebug.LogUpdateBegin;
begin
  lvDetails.Items.BeginUpdate;

end;

procedure TfrmDebug.LogUpdateEnd;
begin
  lvDetails.Items.EndUpdate;
end;

procedure TfrmDebug.LogUpdate(aText: string);
begin
  mmDetails.Lines.Add(aText);
end;

procedure TfrmDebug.LogUpdate(aType, aName, aValue: string);
var
  li: TListItem;
begin
  li := lvDetails.Items.Add;
  li.Caption := aType;
  li.SubItems.Add(aName);
  li.SubItems.Add(aValue);
end;

procedure TfrmDebug.LogClear;
begin
  mmDetails.Lines.Clear;
  lvDetails.Items.Clear;
end;

procedure TfrmDebug.Clear;
begin
  LogClear;
  tv.Items.Clear;
end;

procedure TfrmDebug.acAlignLeftExecute(Sender: TObject);
begin
  inherited;
  AlignForm(self, alLeft);
end;

procedure TfrmDebug.acAlignRightExecute(Sender: TObject);
begin
  inherited;
  AlignForm(self, alRight);
end;

procedure TfrmDebug.acAllTypesExecute(Sender: TObject);
begin
  inherited;
  doGetTypes;
end;

procedure TfrmDebug.acAutoRefreshExecute(Sender: TObject);
begin
  inherited;
  acAutoRefresh.Checked := not acAutoRefresh.Checked;
  tmRTTI.Enabled := acAutoRefresh.Checked;
end;

procedure TfrmDebug.acCloseExecute(Sender: TObject);
begin
  inherited;
  Close;
end;

procedure TfrmDebug.acViewExecute(Sender: TObject);
begin
  inherited;
  acView.Checked := not acView.Checked;
  mmDetails.Visible := acView.Checked;
  lvDetails.Visible := not acView.Checked;
end;

procedure TfrmDebug.AddObject(anObject: TObject; aName: String = '');
var
  sName: String;
  b: Boolean;

  function FindObject(anObject: TObject): TTreeNode;
  var
    tn: TTreeNode;
  begin
    Result := nil;
    for tn in tv.Items do
    begin
      if tn.Data = Pointer(anObject) then
      begin
        Result := tn;
        break
      end;
    end;

    if Result <> nil then
      tv.Selected := Result;
  end;

begin
  if not assigned(anObject) then
    exit;

  if FindObject(anObject) <> nil then
    exit;

  if trim(aName) = '' then
  begin
    sName := getObjectFieldByName(anObject, 'Name');
    sName := sName + ' (' + anObject.ClassName + ', ' +
      IntToStr(Integer(@anObject)) + ')';
  end
  else
    sName := aName;
  tv.Enabled := False;
  b := _Ignore;
  _Ignore := True;
  tv.Items.AddObject(nil, sName, anObject);
  _Ignore := b;
  tv.Enabled := True;
end;
(*
procedure TfrmDebug.AddObjectList(aList: TStringList; aName: String;
  addNils: Boolean = False);
var
  s: String;
  i: Integer;
  obj: TObject;
begin
  for i := 0 to aList.Count - 1 do
  begin
    s := aList[i];
    obj := TObject(aList.Objects[i]);
    if (obj <> nil) or addNils then
      AddObject(obj, s);
  end;
end;
*)
/// /////////////////////////////////////////////////////////////////////////////

procedure TfrmDebug.doGetTypes;
var
  aContext: TRttiContext;
  theTypes: TArray<TRttiType>;
  aType: TRttiType;
  tn: TTreeNode;

  procedure addNode(aName: String);
  begin
    tn := tv.Items.AddChild(ndTypes, aName);
    doTVChange(tn);
  end;

begin
  if not assigned(ndTypes) then
    ndTypes := tv.Items.Add(nil, 'Types');
  ndTypes.DeleteChildren;
  try
    theTypes := aContext.GetTypes;
    for aType in theTypes do
      addNode(aType.QualifiedName);
  finally

  end;
end;

procedure TfrmDebug.RefreshNode(aNode: TTreeNode);
begin
  WriteObjectRTTI(aNode);
end;

initialization

oList := TList.Create;

finalization

oList.Free;

end.
