unit fGN_PtSel;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ORCtrls, ORDtTmRng, ExtCtrls, ComCtrls, Buttons, ORFn,
  rGN_Core, ORNet, ORDtTm, fGN_Page, fGN_MainFrame, uCore, uPCE, fPtSelDemog,
  uTIU, System.Actions, Vcl.ActnList, fGN_GroupEncounter, fPtSelOptns,
  fGN_SelectedPatients, oGN_PtData, Vcl.ToolWin,
  U508Extensions, VA508AccessibilityManager
  ;

type
  TSetPtListTopProc = procedure(IEN: Int64) of object;

  TfrmGNPtSel = class(TfrmPage)
    pnlTop: TPanel;
    pnlOptions: TPanel;
    pnlRight: TPanel;
    splPatients: TSplitter;
    pnlPtSel: TPanel;
    cboPatient: TORComboBox;
    pnlOptionsHeader: TPanel;
    lblOptions: TLabel;
    pnlPatientHeader: TPanel;
    calApptRng: TORDateRangeDlg;
    pnlSelectedPatients: TPanel;
    pnlDemographics: TPanel;
    pnlGroupEncounter: TPanel;
    alPatientSelector: TActionList;
    acInquiry: TAction;
    pnlButtons: TPanel;
    cmdOK: u508Extensions.TButton;
    cmdCancel: u508Extensions.TButton;
    pnlGroupEncounterHeader: TPanel;
    lblGroupEncounter: TLabel;
    acOneClickSelection: TAction;
    CheckBox1: TCheckBox;
    pnlSelect: TPanel;
    acSelect: TAction;
    lblPatient: TLabel;
    bbSelect: U508Extensions.TButton;
    bbInquiry: U508Extensions.TButton;
    procedure cboPatientNeedData(Sender: TObject; const StartFrom: String;
      Direction, InsertAt: Integer);
    procedure cboPatientChange(Sender: TObject);
    procedure cboPatientClick(Sender: TObject);
    procedure cboPatientKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormShow(Sender: TObject);
    procedure cboPatientDblClick(Sender: TObject);
    procedure acInquiryExecute(Sender: TObject);
    procedure pnlTopResize(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure acOneClickSelectionExecute(Sender: TObject);
    procedure acSelectExecute(Sender: TObject);
    procedure ToolButton2Click(Sender: TObject);
    procedure cboPatientExit(Sender: TObject);
    procedure cboPatientEnter(Sender: TObject);
    procedure cboPatientKeyPause(Sender: TObject);
    procedure cboPatientKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure cboPatientMouseClick(Sender: TObject);
    procedure cboPatientDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure FormHide(Sender: TObject);
  private
    fOneClickSelection: boolean;
    FLastTopList: string;
    FSrcType: Integer;
    FSetPtListTop: TSetPtListTopProc;
    FDfltSrc: string;
    FDfltSrcType: string;
    FLastDFN: string;
    fInside: Boolean;

    _Started: boolean;
    _GroupEncounter: TfrmGN_GroupEncounter;
    _GroupDemographics: TfrmPtSelDemog;
    _GroupOptions: TfrmPtSelOptns;
    _SelectedPatients: TfrmGN_SelectedPatients;

    procedure ShowDemog(ItemID: string);
    procedure AddToGNPtList(APtData: TPatientData);
    procedure ClearIDInfo;

    function DupLastSSN(var DFN: string): TStringList;
    procedure DoInquiry;
    procedure DoAdd(anID,aName: String);

    procedure InitView;
    function MoveOnAllowed(aStatus: Integer; aMsg: String = ''): boolean;
    function ContinueOnDeceased(aDfn: string): boolean;

    procedure ShowComponentWindows;

  public
    property OneCLick: boolean read fOneClickSelection write fOneClickSelection;
    property LastTopList: string read FLastTopList write FLastTopList;
    property SrcType: Integer read FSrcType write FSrcType;
    property SetPtListTopProc: TSetPtListTopProc read FSetPtListTop
      write FSetPtListTop;
    procedure SetCaptionTop;
    procedure SetPtListTop(IEN: Int64);
    procedure setEncounter;

    procedure setLayout(aLayout: String);

    function isComplete: boolean;
    procedure focusIncomplete;

    procedure setActionStatus;
    function PatientsCount: Integer;
    procedure SetFontSize(NewFontSize: Integer); override;
  end;

var
  frmGNPtSel: TfrmGNPtSel;
  IsRPL, RPLJob, DupDFN: string;
  RPLProblem: boolean;
  FDfltSrc, FDfltSrcType: string;

procedure SelectPatients(aLayout: String = '');

implementation

{$R *.dfm}

uses
  fGN_Encounter, uGN_Utils, rPCE, rCore, uConst, uGN_Const, fRptBox,
  dmGN_Common, fDupPts, VA508AccessibilityRouter;

var
  _frm: TfrmGNPtSel;

const
  fmtERR_Duplicate = 'Patient %s already selected';
  AliasString = ' -- ALIAS';

procedure SelectPatients(aLayout: String = '');
{ patient selection dialog }
begin
  if not assigned(_frm) then
    Application.CreateForm(TfrmGNPtSel,_frm);
  _frm.BorderStyle := bsSizeable;
  _frm.Position := poScreenCenter;
  _frm.pnlButtons.Visible := True;
  _frm._SelectedPatients.loadFromList(GNPtList);
  _frm.setLayout(aLayout);
  _frm.SetFontSize(Application.MainForm.Font.Size);
  _frm.ShowModal;
  FReeAndNil(_frm);
end;

function DupPtInList(aList: TList; APtDFN: string): Boolean;
var
  i: Integer;
  APatient: TPatientData;
begin
  Result := False;
  for i := 0 to (aList.Count - 1) do
  begin
    APatient := TPatientData(aList.Items[i]);
    if APatient.Patient.DFN = APtDFN then
    begin
      Result := True;
      Break;
    end;
  end;
end;

/// /////////////////////////////////////////////////////////////////////////////

procedure TfrmGNPtSel.ClearIDInfo;
{ clear demographic info }
begin
  FLastDFN := '';
  acInquiry.Enabled := False;
  _GroupDemographics.LastDFN := '';
end;

procedure TfrmGNPtSel.InitView;

  procedure SetupOptions;
  begin
    _GroupOptions := TfrmPtSelOptns.Create(self); // Was application - kcm
    setFormParented(_GroupOptions, pnlOptions);
    with _GroupOptions do
    begin
      SetCaptionTopProc := SetCaptionTop;
      SetPtListTopProc := SetPtListTop;
      TabOrder := 0;
    end;
  end;

  procedure setupSelectedList;
  begin
    if not assigned(GNPtList) then
      GNPtList := TList.Create;
    if not assigned(_SelectedPatients) then
      _SelectedPatients := TfrmGN_SelectedPatients.Create(self);
    setFormParented(_SelectedPatients, pnlSelectedPatients);
  end;

  procedure setDemographics;
  begin
    if not assigned(_GroupDemographics) then
      _GroupDemographics := TfrmPtSelDemog.Create(self);
    setFormParented(_GroupDemographics, pnlDemographics);
    _GroupDemographics.gpDemog.Visible := not ScreenReaderSystemActive;
  end;

  procedure setGroupEncounter;
  begin
    if not assigned(_GroupEncounter) then
      _GroupEncounter := TfrmGN_GroupEncounter.Create(self);
    setFormParented(_GroupEncounter, pnlGroupEncounter);
    pnlGroupEncounter.Height := _GroupEncounter.getHeight;

    if assigned(Encounter) and (Encounter.Provider > 0) then
      _GroupEncounter.cboPtProvider.SelectByIEN(Encounter.Provider);
  end;

begin
  SetupOptions;
  setDemographics;
  setGroupEncounter;
  setupSelectedList;

  RPLProblem := False;
  FDfltSrc := DfltPtList;
  FDfltSrcType := Piece(FDfltSrc, U, 2);
  FDfltSrc := Piece(FDfltSrc, U, 1);

  if (IsRPL = '1') then // Deal with restricted patient list users.
    FDfltSrc := '';

  ClearIDInfo;

  if RPLProblem then
    exit; // No code found that set RPLProblem to True

  if (IsRPL = '1') then // Deals with restricted patient list users.
    pnlOptions.Visible := False; // Removes unnecessary components from view.

  cboPatient.InitLongList('');

  if assigned(_GroupEncounter) then
    _GroupEncounter.Init;

  _GroupOptions.rgPtList.ItemIndex := 0;

  setActionStatus;

  OneCLick := acOneClickSelection.Checked;

  _Started := True;

  ShowComponentWindows;

  pnlOptionsHeader.Visible := not ScreenReaderSystemActive;
  pnlGroupEncounterHeader.Visible := not ScreenReaderSystemActive;

  if ScreenReaderSystemActive then
    begin
      _GroupOptions.rgPtList.Caption := 'Search Categories';
      _GroupEncounter.gb.Caption := 'Group Encounter';
    end
  else
    begin
      _GroupOptions.rgPtList.Caption := '';
      _GroupEncounter.gb.Caption := '';
    end;
end;

procedure TfrmGNPtSel.FormCreate(Sender: TObject);
begin
  inherited;
  InitView;

  Height := pnlTop.Height + pnlSelectedPatients.Height;
  if pnlButtons.Visible then
    Height := Height + pnlButtons.Height;

  if assigned(Application.MainForm) then
    setFontSize(Application.MainForm.Font.Size);
end;

procedure TfrmGNPtSel.FormHide(Sender: TObject);
begin
  inherited;
  _SelectedPatients.acRemove.Enabled := False;
  _SelectedPatients.acRemoveAll.Enabled := False;
end;

procedure TfrmGNPtSel.FormResize(Sender: TObject);
begin
  inherited;

  if pnlTop.Height > Round(Height * 4 / 5) then
    pnlTop.Height := Round(Height * 4 / 5);
end;
(*
procedure TfrmGNPtSel.cboPatientNeedData(Sender: TObject;
  const StartFrom: string; Direction, InsertAt: Integer);
{ process component request for data }
var
  i: Integer;
  NoAlias, ThePatient: String;
  PatientList: TStringList;

const
  ALIASSTRING = ' -- ALIAS';

begin
  NoAlias := StartFrom;
  with Sender as TORComboBox do
  begin
    if Items.Count > ShortCount then
      NoAlias := Piece(Items[Items.Count - 1], U, 1) + U + NoAlias;
  end;
  if pos(ALIASSTRING, NoAlias) > 0 then
    NoAlias := Copy(NoAlias, 1, pos(ALIASSTRING, NoAlias) - 1);
  PatientList := TStringList.Create;
  try
    begin
      if (IsRPL = '1') then
        PatientList.Assign(ReadRPLPtList(RPLJob, NoAlias, Direction))
      else
      begin
        PatientList.Assign(SubSetOfPatients(NoAlias, Direction));
        for i := 0 to PatientList.Count - 1 do
        begin
          ThePatient := PatientList[i];
          if (Uppercase(Piece(ThePatient, U, 2)) <> Uppercase(Piece(ThePatient,
            U, 6))) then
          begin
            SetPiece(ThePatient, U, 2, Piece(ThePatient, U, 2) + ALIASSTRING);
            PatientList[i] := ThePatient;
          end;
        end;
      end;
      cboPatient.ForDataUse(PatientList);
    end;
  finally
    PatientList.Free;
  end;
end;
*)
procedure TfrmGNPtSel.cboPatientNeedData(Sender: TObject;
  const StartFrom: string; Direction, InsertAt: Integer);

//procedure TfrmPtSel.cboPatientNeedData(Sender: TObject; const StartFrom: string;
//  Direction, InsertAt: Integer);
var
  i: Integer;
  NoAlias, Patient: string;
  PatientList: TStringList;
begin
  NoAlias := StartFrom;
  with Sender as TORComboBox do
    if Items.Count > ShortCount then
      begin
        NoAlias := Piece(Items[Items.Count - 1], U, 1) + U + NoAlias;
        if Direction < 0 then
          NoAlias := Copy(NoAlias, 1, Length(NoAlias) - 1);
      end;
  if pos(AliasString, NoAlias) > 0 then
    NoAlias := Copy(NoAlias, 1, pos(AliasString, NoAlias) - 1);
  PatientList := TStringList.Create;
  try
    begin
      if (IsRPL = '1') then // Restricted patient lists uses different feed for long list box:
        setRPLPtList(PatientList, RPLJob, NoAlias, Direction)
      else
        begin
          setSubSetOfPatients(PatientList, NoAlias, Direction);
          for i := 0 to PatientList.Count - 1 do // Add " - Alias" to alias names:
            begin
              Patient := PatientList[i];
              // Piece 6 avoids display problems when mixed with "RPL" lists:
              if (Uppercase(Piece(Patient, U, 2)) <> Uppercase(Piece(Patient, U, 6))) then
                begin
                  SetPiece(Patient, U, 2, Piece(Patient, U, 2) + AliasString);
                  PatientList[i] := Patient;
                end;
            end;
        end;
      cboPatient.ForDataUse(PatientList);
    end;
  finally
    PatientList.Free;
  end;
end;

procedure TfrmGNPtSel.ShowDemog(ItemID: string);
{ gets a record of patient indentifying information from the server and displays it }
begin
  if ItemID = FLastDFN then
    exit;
  if (StrToInt64Def(ItemID, 0) < 1) then
    exit;
  FLastDFN := ItemID;
  if assigned(_GroupDemographics) then
    _GroupDemographics.LastDFN := ItemID;
  acInquiry.Enabled := True;
end;

procedure TfrmGNPtSel.ToolButton2Click(Sender: TObject);
begin
  inherited;
  if frmGNMainFrame.acDebugLayout.Checked then
    ModalResult := mrOK
  else
    frmGNMainFrame.acNewGroupNote.Execute;
end;

procedure TfrmGNPtSel.cboPatientChange(Sender: TObject);

  procedure ShowMatchingPatients;
  begin
    with cboPatient do
      begin
        ClearIDInfo;
        if ShortCount > 0 then
          begin
            if ShortCount = 1 then
              begin
                ItemIndex := 0;
//                ShowIDInfo;
//                ShowFlagInfo;
          ShowDemog(cboPatient.ItemID);
              end;
            Items.Add(LLS_LINE);
            Items.Add(LLS_SPACE);
          end;
        InitLongList('');
      end;
  end;

begin
  with cboPatient do
    if {frmPtSelOptns.}IsLast5(Text) then
      begin
        if (IsRPL = '1') then
          ListPtByRPLLast5(Items, Text)
        else
          ListPtByLast5(Items, Text);
        ShowMatchingPatients;
      end
    else if {frmPtSelOptns.}IsFullSSN(Text) then
      begin
        if (IsRPL = '1') then
          ListPtByRPLFullSSN(Items, Text)
        else
          ListPtByFullSSN(Items, Text);
        ShowMatchingPatients;
      end;

  acSelect.Enabled := cboPatient.ItemIndex >= 0;
end;

function TfrmGNPtSel.DupLastSSN(var DFN: string): TStringList; // boolean;
{ check for duplacate SSN }
var
  i: Integer;
  PtStrs: TStringList;
begin
  Result := nil;
  CallV('DG CHK BS5 XREF ARRAY', [DFN]);
  if (RPCBrokerV.Results[0] <> '1') then
    exit;
  PtStrs := TStringList.Create;
  with RPCBrokerV do
    if Results.Count > 1 then
    begin
      for i := 1 to Results.Count - 1 do
      begin
        if Piece(Results[i], U, 1) = '1' then
          PtStrs.Add(Piece(Results[i], U, 2) + U + Piece(Results[i], U, 3) + U +
            FormatFMDateTimeStr('mmm dd,yyyy', Piece(Results[i], U, 4)) + U +
            Piece(Results[i], U, 5));
      end;
    end;

  Result := PtStrs;

  // SelectDupPt(DFN, PtStrs) <> mrOK;
end;

const
  TX_DGSR_ERR = 'Unable to perform sensitive record checks';
  TC_DGSR_ERR = 'Error';
  TC_DGSR_SHOW = 'Restricted Record';
  TC_DGSR_DENY = 'Access Denied';
  TX_DGSR_YESNO = CRLF +
    'Do you want to continue processing this patient record?';

function TfrmGNPtSel.MoveOnAllowed(aStatus: Integer; aMsg: String = '')
  : boolean;
var
  NewDFN: string;
begin
  Result := False;
  case aStatus of
    DGSR_FAIL:
      InfoBox(TX_DGSR_ERR, TC_DGSR_ERR, MB_OK);

    DGSR_NONE: { Nothing - allow access to the patient. }
      Result := True;
    DGSR_SHOW:
      begin
        InfoBox(aMsg, TC_DGSR_SHOW, MB_OK);
        Result := True;
      end;
    DGSR_ASK:
      if InfoBox(aMsg + TX_DGSR_YESNO, TC_DGSR_SHOW,
        MB_YESNO or MB_ICONWARNING or MB_DEFBUTTON2) = IDYES then
      begin
        LogSensitiveRecordAccess(NewDFN);
        acInquiry.Enabled := True;
        Result := True;
      end;
  else
    InfoBox(aMsg, TC_DGSR_DENY, MB_OK);
  end;
end;

function TfrmGNPtSel.ContinueOnDeceased(aDfn: string): boolean;
var
  DateDied: TFMDateTime;
begin
  Result := True;
  DateDied := DateOfDeath(aDfn);
  if (DateDied > 0) and
    (InfoBox('This patient died ' + FormatFMDateTime('mmm dd,yyyy hh:nn',
    DateDied) + CRLF + 'Do you wish to continue?', 'Deceased Patient',
    MB_YESNO or MB_DEFBUTTON2) = ID_NO) then
    Result := False;
end;

procedure TfrmGNPtSel.cboPatientClick(Sender: TObject);
{ check access to selected patient, process selection if allowed }

var
  NewDFN, aName, aMsg: string;
  AccessStatus: Integer;

begin
  // see also OnChange

  Enabled := False;
  Application.ProcessMessages;
  Enabled := True;

  NewDFN := cboPatient.ItemID;
  if StrToInt64Def(NewDFN, 0) >= 0 then
  begin
    Enabled := False;
    aName := cboPatient.Items[cboPatient.ItemIndex];

    CheckSensitiveRecordAccess(NewDFN, AccessStatus, aMsg);
    if MoveOnAllowed(AccessStatus) then
      if OneCLick then
        DoAdd(NewDFN,aName);

    Enabled := True;
  end;
end;

procedure TfrmGNPtSel.cboPatientDblClick(Sender: TObject);
begin
  inherited;
  if OneCLick then // versions before OR*3*455 supported one click selection
    acSelectExecute(Sender)
  else
    DoInquiry;
end;

procedure TfrmGNPtSel.cboPatientDrawItem(Control: TWinControl; Index: Integer;
  Rect: TRect; State: TOwnerDrawState);
var
  Offset: Integer;
begin
  with (Control as TListBox) do
  begin
    if (odSelected in State) then
    begin
      if not fInside then
        begin
          Canvas.Brush.Color := clSilver;
          Canvas.Pen.Color := clSilver;
          Canvas.Font.Color := clWindowText;
        end
      else
        Canvas.Font.Color := clWhite;
    end
    else
    begin
      Canvas.Pen.Color := Color;
      Canvas.Brush.Color := Color;
      Canvas.Font.Color := clWindowText;
    end;

    Canvas.FillRect(Rect);
    Canvas.Rectangle(Rect);

    Offset := (Rect.Bottom - Rect.Top - Canvas.TextHeight(Items[Index])) div 2;
    Canvas.TextRect(Rect, Rect.Left + Offset + 2, Rect.Top + Offset, Items[Index]);
  end;
end;

procedure TfrmGNPtSel.cboPatientEnter(Sender: TObject);
begin
  cmdOK.Default := True;
  if cboPatient.ItemIndex >= 0 then
    begin
      ShowDemog(cboPatient.ItemID);
//      ShowFlagInfo; -- ???
    end;
  fInside := True;
end;

procedure TfrmGNPtSel.cboPatientExit(Sender: TObject);
begin
  cmdOK.Default := False;
  fInside := False;
end;

procedure TfrmGNPtSel.DoAdd(anID,aName: string);
{ add patient data to the list of selected patients }
{ check for duplicates, notify the application main form on selection }
var
  APtData: TPatientData;
  sl: TStringList;
begin
  if not DupPtInList(GNPtList, anID) then // check if the pt was selected before
  begin
    sl := DupLastSSN(anID); // get list of patients with 'duplicate SSN'

    if (sl = nil) or SelectDupPt(anID, sl) then // select one pt if needed
      if not DupPtInList(GNPtList, anID) then // have to check again
      begin
        APtData := TPatientData.Create;
        APtData.PCEData := TPCEData.Create;
        APtData.Patient.DFN := anID;
//        APtData.ptName := aName;
        AddToGNPtList(APtData);
        FLastDFN := anID;
        sendMessage(Application.MainForm.Handle, UM_GNSELECT, 0, 0);

        if ScreenReaderSystemActive then
          GetScreenReader.Speak('Patient added to selected patients list');
      end;
    if assigned(sl) then
      sl.Free;
  end;
  setActionStatus;
  Keybd_Event(VK_Down,0,0,0);
end;

procedure TfrmGNPtSel.acInquiryExecute(Sender: TObject);
{ invoke event processing routine }
begin
  inherited;
  DoInquiry;
end;

procedure TfrmGNPtSel.acOneClickSelectionExecute(Sender: TObject);
begin
  inherited;
  acOneClickSelection.Checked := not acOneClickSelection.Checked;
  OneCLick := acOneClickSelection.Checked;
end;

procedure TfrmGNPtSel.acSelectExecute(Sender: TObject);
var
  sName,
  NewDFN, aMsg: string;
  AccessStatus: Integer;
begin
  NewDFN := cboPatient.ItemID;
  if StrToInt64Def(NewDFN, -1) >= 0 then
  begin
    if DupPtInList(GNPtList, NewDFN) then
    begin
      aMsg := Piece(cboPatient.Items[cboPatient.ItemIndex], U, 2);
      InfoBox(Format(fmtERR_Duplicate, [aMsg]),'Error',MB_OK + MB_ICONERROR);
    end
    else
    begin
      CheckSensitiveRecordAccess(NewDFN, AccessStatus, aMsg);
      if MoveOnAllowed(AccessStatus, aMsg) and ContinueOnDeceased(NewDFN) then
        begin
          sName := cboPatient.Items[cboPatient.ItemIndex];
          DoAdd(NewDFN,sName);
          if assigned(_GroupDemographics) then
            _GroupDemographics.ShowDemog(NewDFN);
//          if ScreenReaderSystemActive then
//            GetScreenReader.Speak('Patient added to selected list');
        end;
    end;
  end;
end;

procedure TfrmGNPtSel.AddToGNPtList(APtData: TPatientData);
{ add PtData to Selected patients list and register it in the global list }
begin
  if assigned(_SelectedPatients) and assigned(_GroupDemographics) then
    _SelectedPatients.addPatientData(APtData,
      _GroupDemographics.lblPtLocation.Caption);

  GNPtList.Add(APtData);
end;

procedure TfrmGNPtSel.DoInquiry;
{ present Patient demographic info in report box }
var
  sl: TStringList;
begin
  if cboPatient.ItemID <> '' then
  begin
    sl := TStringList.Create;
    try
      LoadDemographics(sl, IntToStr(cboPatient.ItemID));
      ReportBox(sl, 'Patient Inquiry', True);
    finally
      sl.Free;
    end;
  end;
end;

procedure TfrmGNPtSel.cboPatientKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
{ processing RETURN key press }
begin
  inherited;
  if Key = VK_RETURN then
  begin
    if ssShift in Shift then
      acInquiry.Execute
    else
      cboPatientDblClick(Sender);
    Key := 0;
  end;
end;

procedure TfrmGNPtSel.FormShow(Sender: TObject);
{ reset patient list }
begin
  inherited;
  if StartOver and assigned(_SelectedPatients) then
    _SelectedPatients.lvGnPtList.Items.Clear;

  _SelectedPatients.setActionStatus;

//  ShowComponentWindows;
//
//  pnlTop.Height := pnlTop.Height + 1;
//  Application.ProcessMessages;
//  pnlTop.Height := pnlTop.Height - 1;
//  setFontSize(Application.MainForm.Font.Size);
end;

procedure TfrmGNPtSel.SetCaptionTop;
{ Show patient list name, set top list to 'Select ...' if appropriate. }
var
  X: string;
begin
  X := '';
  lblPatient.Caption := '&Patients';
  if (not User.IsReportsOnly) then
  begin
    case _GroupOptions.SrcType of
      TAG_SRC_DFLT:
        if FDfltSrc <> '' then
          lblPatient.Caption := '&Patients (' + FDfltSrc + ')'
        else
          lblPatient.Caption := '&Patients';
      TAG_SRC_PROV:
        X := 'Provider';
      TAG_SRC_TEAM:
        X := 'Team';
      TAG_SRC_SPEC:
        X := 'Specialty';
      TAG_SRC_CLIN:
        X := 'Clinic';
      TAG_SRC_WARD:
        X := 'Ward';
      TAG_SRC_PCMM:
        X := 'PCMM Team'; // TDP - Added 5/27/2014 to handle PCMM team addition
      // TAG_SRC_ALL: { Nothing }
      // ;
    end; // case stmt
  end; // begin
  if Length(X) > 0 then
    with cboPatient do
    begin
      RedrawSuspend(Handle);
      ClearIDInfo;
      ClearTop;
      Text := '';
      Items.Add('^Select a ' + X + '...');
      Items.Add(LLS_LINE);
      Items.Add(LLS_SPACE);
      cboPatient.InitLongList('');
      RedrawActivate(cboPatient.Handle);
    end;
end;

{ List Source events: }

procedure TfrmGNPtSel.SetPtListTop(IEN: Int64);
{ Sets top items in patient list according to list source type and optional list source IEN. }
var
  updateDate: String;
  NewTopList: string;
  FirstDate, LastDate: string;
begin
  // NOTE:  Some pieces in RPC returned arrays are rearranged by ListPtByDflt call in rCore!
  IsRPL := User.IsRPL;
  if (IsRPL = '') then
  // First piece in ^VA(200,.101) should always be set (to 1 or 0).
  begin
    InfoBox('Patient selection list flag not set.',
      'Incomplete User Information', MB_OK);
    RPLProblem := True;
    exit;
  end;
  // FirstDate := 0; LastDate := 0; // Not req'd, but eliminates hint.
  // Assign list box TabPosition, Pieces properties according to type of list to be displayed.
  // (Always use Piece "2" as the first in the list to assure display of patient's name.)
  cboPatient.pieces := '2,3';
  // This line and next: defaults set - exceptions modifield next.
  cboPatient.tabPositions := '20,28';
  if ((_GroupOptions.SrcType = TAG_SRC_DFLT) and (FDfltSrc = 'Combination'))
  then
  begin
    cboPatient.pieces := '2,3,4,5,9';
    cboPatient.tabPositions := '20,28,35,45';
  end;
  if ((_GroupOptions.SrcType = TAG_SRC_DFLT) and (FDfltSrcType = 'Ward')) or
    (_GroupOptions.SrcType = TAG_SRC_WARD) then
    cboPatient.tabPositions := '35';
  if ((_GroupOptions.SrcType = TAG_SRC_DFLT) and
    (AnsiStrPos(pChar(FDfltSrcType), 'Clinic') <> nil)) or
    (_GroupOptions.SrcType = TAG_SRC_CLIN) then
  begin
    cboPatient.pieces := '2,3,9';
    cboPatient.tabPositions := '24,45';
  end;
  NewTopList := IntToStr(_GroupOptions.SrcType) + U + IntToStr(IEN);
  // Default setting.
  if (_GroupOptions.SrcType = TAG_SRC_CLIN) then
    with _GroupOptions.cboDateRange do
    begin
      if ItemID = '' then
        exit; // Need both clinic & date range.
      FirstDate := Piece(ItemID, ';', 1);
      LastDate := Piece(ItemID, ';', 2);
      NewTopList := IntToStr(_GroupOptions.SrcType) + U + IntToStr(IEN) + U +
        ItemID; // Modified for clinics.
    end;
  if NewTopList = _GroupOptions.LastTopList then
    exit; // Only continue if new top list.
  _GroupOptions.LastTopList := NewTopList;
  RedrawSuspend(cboPatient.Handle);
  ClearIDInfo;
  cboPatient.ClearTop;
  cboPatient.Text := '';
  if (IsRPL = '1') then // Deal with restricted patient list users.
  begin
    RPLJob := MakeRPLPtList(User.RPLList);
    // MakeRPLPtList is in rCore, writes global "B" x-ref list.
    if (RPLJob = '') then
    begin
      InfoBox('Assignment of valid OE/RR Team List Needed.',
        'Unable to build Patient List', MB_OK);
      RPLProblem := True;
      exit;
    end;
  end
  else
  begin
    case _GroupOptions.SrcType of
      TAG_SRC_DFLT:
        ListPtByDflt(cboPatient.Items);
      TAG_SRC_PROV:
        ListPtByProvider(cboPatient.Items, IEN);
      TAG_SRC_TEAM:
        ListPtByTeam(cboPatient.Items, IEN);
      TAG_SRC_SPEC:
        ListPtBySpecialty(cboPatient.Items, IEN);
      TAG_SRC_CLIN:
        ListPtByClinic(cboPatient.Items, _GroupOptions.cboList.ItemIEN,
          FirstDate, LastDate);
      TAG_SRC_WARD:
        ListPtByWard(cboPatient.Items, IEN);
      // TDP - Added 5/27/2014 to handle PCMM team addition
      TAG_SRC_PCMM:
        ListPtByPcmmTeam(cboPatient.Items, IEN);
      TAG_SRC_ALL:
        ListPtTop(cboPatient.Items);
    end;
  end;
  if _GroupOptions.cboList.Visible then
//    lblPatient.Caption := '&Patients (' + _GroupOptions.cboList.Text + ')';
///////////////////////////////////////////////////////// - added to build 260+
    with _GroupOptions.cboList do
    begin
      updateDate := '';
      if ItemIndex < 0 then
        lblPatient.Caption := '&Patients (' + Text + ')'
      else if (_GroupOptions.SrcType <> TAG_SRC_PROV) and
         (Piece(Items[ItemIndex], U, 3) <> '') then
        updateDate := ' last updated on ' + Piece(Items[ItemIndex], U, 3);
      lblPatient.Caption := '&Patients (' + Text + updateDate + ')';
    end;
///////////////////////////////////////////////////////// - added to build 260+
  if _GroupOptions.SrcType = TAG_SRC_ALL then
    lblPatient.Caption := '&Patients (All Patients)';
  with cboPatient do
    if ShortCount > 0 then
    begin
      Items.Add(LLS_LINE);
      Items.Add(LLS_SPACE);
    end;
  cboPatient.Caption := lblPatient.Caption;
  cboPatient.InitLongList('');
  RedrawActivate(cboPatient.Handle);
end;

procedure TfrmGNPtSel.pnlTopResize(Sender: TObject);
{ adjust size of the Group encounter info area }
begin
  if assigned(_GroupEncounter) then
    pnlGroupEncounter.Height := _GroupEncounter.getHeight;
end;

procedure TfrmGNPtSel.setEncounter;
{ update global uCore.Encounter properties based on the selected values }
begin
  if assigned(_GroupEncounter) then
    _GroupEncounter.setEncounter;
  Font.Size := Application.MainForm.Font.Size;
end;

procedure TfrmGNPtSel.setLayout(aLayout: string);
begin
  _SelectedPatients.tbSelectedPatients.Visible := aLayout = 'DEBUG';
  _SelectedPatients.bbRemove.Visible := not(aLayout = 'DEBUG');
  _SelectedPatients.bbRemoveAll.Visible := not(aLayout = 'DEBUG');
end;

function TfrmGNPtSel.isComplete: boolean;
begin
  Result := _GroupEncounter.isComplete;
  focusIncomplete;
end;

procedure TfrmGNPtSel.setActionStatus;
begin
  if assigned(_SelectedPatients) then
    cmdOK.Enabled := _SelectedPatients.lvGnPtList.Items.Count > 0;
end;

function TfrmGNPtSel.PatientsCount: Integer;
begin
  Result := 0;
  if assigned(_SelectedPatients) then
    Result := _SelectedPatients.lvGnPtList.Items.Count;
end;

procedure TfrmGNPtSel.SetFontSize(NewFontSize: Integer);
var
  i: Integer;
begin
  inherited;
  if Font.Size = NewFontSize then
    exit;

  Font.Size := NewFontSize;

  pnlSelect.Height := 24 + // default margins for panel and buttons
    getMainFormTextHeight;

  cboPatient.ItemHeight := getMainFormTextHeight + 2;
//  adjustBitBtn(bbSelect);
//  adjustBitBtn(bbInquiry);
  adjustBtn(bbSelect);
  adjustBtn(bbInquiry);

  i := bbSelect.Width + bbSelect.Margins.Left + bbSelect.Margins.Right;
  i := i + bbInquiry.Width + bbInquiry.Margins.Left + bbInquiry.Margins.Right;

  if pnlRight.Width < i then
    pnlRight.Width := i;

  i := getMainFormTextHeight + lblOptions.Margins.Top +
    lblOptions.Margins.Bottom;

  lblOptions.Font.Size := NewFontSize;
  lblPatient.Font.Size := NewFontSize;
  lblGroupEncounter.Font.Size := NewFontSize;

  pnlOptionsHeader.Height := i;
  pnlPatientHeader.Height := i;
  pnlGroupEncounterHeader.Height := i;

  if assigned(_GroupEncounter) and (_GroupEncounter.Font.Size <> NewFontSize) then
    begin
      _GroupEncounter.Font.Size := NewFontSize;
      pnlGroupEncounter.Constraints.MinHeight := _GroupEncounter.getHeight;
      pnlGroupEncounter.Height := _GroupEncounter.getHeight;
    end;

  if assigned(_GroupDemographics) then
    _GroupDemographics.SetFontSize(NewFontSize);

  if assigned(_GroupOptions) and (_GroupOptions.Font.Size <> NewFontSize)  then
  begin
    _GroupOptions.Font.Size := NewFontSize;
    _GroupOptions.rgPtList.Height := (getMainFormTextHeight + 4) * 4 + 24;
    _GroupOptions.pnlDateRange.Height :=
     (_GroupOptions.cboDateRange.Height + GAP) * 2;

    _GroupOptions.cboList.ItemHeight := getMainFormTextHeight + 2;

    i := (getMainFOrmTextWidth('Team/Personal') + 24 + 6) * 2;
    if i < 202 then
      i := 202;
    pnlOptions.Width := i;
    if pnlOptions.Width < pnlRight.Width then
      pnlOptions.Width := pnlRight.Width;
  end;

  if pnlRight.Width < pnlOptions.Width then
    pnlRight.Width := pnlOptions.Width;

  if assigned(_SelectedPatients) then
  begin
    _SelectedPatients.setFontSize(NewFontSize);
    _SelectedPatients.pnlTools.Height := pnlSelect.Height;

    bbInquiry.Width := _SelectedPatients.bbRemoveAll.Width;
    bbSelect.Width := _SelectedPatients.bbRemove.Width;

    cmdCancel.Width := _SelectedPatients.bbRemoveAll.Width;
    cmdOK.Width := _SelectedPatients.bbRemove.Width;

    _SelectedPatients.lvGnPtList.Columns[0].Width :=
      _SelectedPatients.lvGnPtList.Columns[0].Width + 1;

    pnlSelect.Height := _SelectedPatients.pnlTools.Height;
    _SelectedPatients.Refresh;
  end;

  pnlSelectedPatients.Refresh;
  Application.ProcessMessages;
end;

procedure TfrmGNPtSel.ShowComponentWindows;
begin
  if assigned(_GroupEncounter) then
    _GroupEncounter.Show;
  if assigned(_GroupDemographics) then
    _GroupDemographics.Show;
  if assigned(_GroupOptions) then
    _GroupOptions.Show;
  if assigned(_SelectedPatients) then
    _SelectedPatients.Show;
end;

procedure TfrmGNPtSel.focusIncomplete;
begin
  _GroupEncounter.focusIncomplete;
end;

procedure TfrmGNPtSel.cboPatientKeyPause(Sender: TObject);
begin
  if Length(cboPatient.ItemID) > 0 then // *DFN*
    ShowDemog(cboPatient.ItemID)
  else
    ClearIDInfo;
end;

procedure TfrmGNPtSel.cboPatientKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  inherited;
  if (Key = VK_BACK) and (cboPatient.Text = '') then
    cboPatient.ItemIndex := -1;
end;

procedure TfrmGNPtSel.cboPatientMouseClick(Sender: TObject);
begin
  if Length(cboPatient.ItemID) > 0 then // *DFN*
    ShowDemog(cboPatient.ItemID)
  else
    ClearIDInfo;
end;

end.
