unit fGN_SelectedPatients;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.Buttons, Vcl.StdCtrls, Vcl.ComCtrls,
  Vcl.ExtCtrls, System.Actions, Vcl.ActnList
  , uPCE, WinAPI.CommCtrl
  , oGN_PtData, Vcl.ToolWin, fBase508Form
  , u508Extensions, UCaptionListView508Manager, VA508AccessibilityManager
  ;

type
  TfrmGN_SelectedPatients = class(TfrmBase508Form)
    lblInfo: TLabel;
    lvGnPtList: TListView;
    pnlTools: TPanel;
    alPatientList: TActionList;
    acRemove: TAction;
    acRemoveAll: TAction;
    tbSelectedPatients: TToolBar;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    bbRemove: u508Extensions.TButton;
    bbRemoveAll: u508Extensions.TButton;
    procedure acRemoveExecute(Sender: TObject);
    procedure acRemoveAllExecute(Sender: TObject);
    procedure lvGnPtListCompare(Sender: TObject; Item1, Item2: TListItem;
      Data: Integer; var Compare: Integer);
    procedure lvGnPtListColumnClick(Sender: TObject; Column: TListColumn);
    procedure lvGnPtListChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure FormCreate(Sender: TObject);
    procedure lvGnPtListCustomDrawSubItem(Sender: TCustomListView;
      Item: TListItem; SubItem: Integer; State: TCustomDrawState;
      var DefaultDraw: Boolean);
  private
    { Private declarations }
    FsortCol: integer;
    FsortAscending: boolean;
    procedure Init508;
  public
    { Public declarations }
    procedure setActionStatus;
    procedure addPatientData(aPtData:TPatientData;aLocation:String);
    procedure removePatientFromList(aPatient:Pointer);
    procedure loadFromList(aList:TList);
    procedure SetFontSize(NewFontSize:Integer); override;
  end;

implementation

{$R *.dfm}
uses
  uCore
  , uGN_Const, rGN_Core
  , ORFn
  , dmGN_Common, uGN_Utils
  , VA508AccessibilityRouter, VAUtils
  ;


procedure ListViewClearSortIndicator(aHandle:HWND; aColumn: Integer);
var
  Header: HWND;
  Item: THDItem;
begin
  Header := ListView_GetHeader(aHandle);
  ZeroMemory(@Item, SizeOf(Item));
  Item.Mask := HDI_FORMAT;

  // Clear the previous arrow
  Header_GetItem(Header, aColumn, Item);
  Item.fmt := Item.fmt and not (HDF_SORTUP or HDF_SORTDOWN);//remove both flags
  Header_SetItem(Header, aColumn, Item);
end;

procedure TfrmGN_SelectedPatients.setActionStatus;
begin
  acRemove.Enabled := (lvGnPtList.Items.Count > 0) and
    (lvGnPtList.ItemIndex >=0) and self.Visible;
  acRemoveAll.Enabled := (lvGnPtList.Items.Count > 0)  and self.Visible;
//
//  sendMessage(Application.MainForm.Handle,UM_GNSELECT,0,0);
//  results in focus change on Pt Selector screen :(
//  will be removed in T14
end;

procedure TfrmGN_SelectedPatients.removePatientFromList(aPatient: Pointer);
var
  i: Integer;
  _PtList: TList;
begin
  _PtList := GNPTList;
  i := _PtList.IndexOf(aPatient);
  if i >= 0 then
  begin
    TPatient(_PtList.Items[i]).Free;
    _PtList.Delete(i);
  end;
end;

procedure TfrmGN_SelectedPatients.acRemoveAllExecute(Sender: TObject);
begin
  if (lvGnPtlist.Items.Count > 0) then
    begin
      lvGnPtList.Items.Clear;
      try
        CleanPtList(GNPtList);
        if ScreenReaderSystemActive then
          GetScreenReader.Speak('Selected patients list is now empty');
      except
        on E: Exception do
          InfoBox(E.Message, 'Error',MB_OK + MB_ICONERROR);
      end;
    end;
  setActionStatus;
  lvGNPtList.SetFocus;
end;

procedure TfrmGN_SelectedPatients.acRemoveExecute(Sender: TObject);
var
  s: String;
  i,ii: integer;
begin
  if lvGNPtList.SelCount <= 0 then
    InfoBox('There is no selected patient', 'Warning', MB_OK or MB_ICONWARNING)
  else
    begin
      ii := 0;
      for i := 0 to lvGNPtList.Items.Count - 1 do
      begin
        if lvGNPtList.Items[i].Selected  then
          begin
            inc(ii);
            removePatientFromList(lvGNPtList.Items[i].Data);
          end;
      end;
      lvGnPtList.DeleteSelected;
      if ScreenReaderSystemActive then
        begin
          if ii < 2 then
            s := 'Selected patient was removed from the list'
          else
            s := 'Selected patients were removed from the list';

          GetScreenReader.Speak(s);
        end;
      setActionStatus;
      lvGNPtList.SetFocus;
    end;
end;

procedure TfrmGN_SelectedPatients.addPatientData(aPtData:TPatientData;aLocation:String);
var
  i,ii: integer;
  AnItem: TListItem;
  PtAdmit: String;
  AdmitDT: TDateTime;
begin
  if not assigned(aPtData) then
    exit;

  lvGnPtList.ShowColumnHeaders := True;

  PtAdmit := '';
  if (APtData.Patient.AdmitTime > 0) then
  begin
    AdmitDT := FMDateTimeToDateTime(APtData.Patient.AdmitTime);
    PtAdmit := DateToStr(AdmitDT);
  end;

  AnItem := lvGNPtList.Items.Add;
  AnItem.Data := APtData;
  AnItem.Caption := APtData.Patient.Name;
  AnItem.SubItems.Add(APtData.Patient.SSN);
  AnItem.SubItems.Add(IntToStr(APtData.Patient.Age));
  AnItem.SubItems.Add(APtData.Patient.Sex);
  AnItem.SubItems.Add(aLocation);
  AnItem.SubItems.Add(PtAdmit);
  AnItem.SubItems.Add(APtData.Patient.PrimaryTeam);
  AnItem.SubItems.Add(APtData.Patient.PrimaryProvider);

  i := 0;
  ii := lvGNPtList.Canvas.TextWidth(anItem.Caption) + GAP + GAP;
  if lvGNPtList.Columns[i].Width < ii then
    lvGNPtList.Columns[i].Width := ii;
  for i := 0 to anItem.SubItems.Count - 1 do
    begin
      ii := lvGNPtList.Canvas.TextWidth(anItem.SubItems[i]) + GAP + GAP;
      if lvGNPtList.Columns[i+1].Width < ii then
        lvGNPtList.Columns[i+1].Width := ii;
    end;
end;

procedure TfrmGN_SelectedPatients.FormCreate(Sender: TObject);
var
  i, ii: Integer;
begin
  inherited;
  Hide;
  FsortAscending := true;
  for i := 0 to lvGNPtList.Columns.Count - 1 do
    begin
      ii := lvGNPtList.Canvas.TextWidth(lvGNPtList.Columns[i].Caption) + gap;
      lvGNPtList.Columns[i].Width := ii;
    end;
  Init508;
end;

procedure TfrmGN_SelectedPatients.Init508;
begin
  if ScreenReaderActive then
      TCaptionListView508Manager.Create(amgrMain, lvGNPtList);
end;

procedure TfrmGN_SelectedPatients.lvGnPtListChange(Sender: TObject;
  Item: TListItem; Change: TItemChange);
begin
  SetActionStatus;
  lvGnPtList.ShowColumnHeaders := lvGnPtList.Items.Count > 0;

end;

procedure TfrmGN_SelectedPatients.lvGnPtListColumnClick(Sender: TObject;
  Column: TListColumn);

  procedure UpdateHeader(aHandle:HWND; aColumn: Integer);
  var
    Header: HWND;
    Item: THDItem;
  begin
    Header := ListView_GetHeader(aHandle);
    ZeroMemory(@Item, SizeOf(Item));
    Item.Mask := HDI_FORMAT;

    ListViewClearSortIndicator(aHandle,FsortCol);

    if Column.Index <> FsortCol then
      begin
        FsortAscending := False;
        FsortCol := Column.Index;
      end
    else
      FsortAscending := not FsortAscending;

    // Get the new column
    Header_GetItem(Header, FsortCol, Item);
    Item.fmt := Item.fmt and not (HDF_SORTUP or HDF_SORTDOWN);//remove both flags

    if FsortAscending then
      Item.fmt := Item.fmt or HDF_SORTDOWN//include the sort descending flag
    else
      Item.fmt := Item.fmt or HDF_SORTUP;//include the sort ascending flag

    Header_SetItem(Header, FsortCol, Item);

    with TListView(Sender) do
      begin
        SortType := stText;
        Items.BeginUpdate;
        AlphaSort;
        Items.EndUpdate;
      end;
  end;

begin
{
  if ((FsortCol = Column.Index))  then
        FsortAscending := not FsortAscending;
  FsortCol := Column.Index;
  lvGnPtList.AlphaSort;
}
  UpdateHeader(TListView(Sender).Handle,FsortCol);
end;

procedure TfrmGN_SelectedPatients.lvGnPtListCompare(Sender: TObject;
  Item1, Item2: TListItem; Data: Integer; var Compare: Integer);
var
  str1, str2: String;
begin
  if not(Sender is TListView) then
    exit;

  if FsortCol > 0 then
  begin
    if Item1.SubItems.Count > (FsortCol - 1) then
      str1 := Item1.SubItems[FsortCol - 1]
    else
      str1 := '';
    if Item2.SubItems.Count > (FsortCol - 1) then
      str2 := Item2.SubItems[FsortCol - 1]
    else
      str2 := '';
  end;

  if FsortAscending then
  begin
    if FsortCol = 0 then
      Compare := CompareStr(Item1.Caption, Item2.Caption)
    else
      Compare := CompareStr(str1, str2);;
  end
  else
  begin
    if FsortCol = 0 then
      Compare := CompareStr(Item2.Caption, Item1.Caption)
    else
      Compare := CompareStr(str2, str1);
  end;
end;

procedure TfrmGN_SelectedPatients.lvGnPtListCustomDrawSubItem(
  Sender: TCustomListView; Item: TListItem; SubItem: Integer;
  State: TCustomDrawState; var DefaultDraw: Boolean);
begin
  inherited;
  if assigned(Item) and (Item.SubItems.Count > 3) then
  begin
    if Item.SubItems[3] <> '' then
      Sender.Canvas.Brush.Color := clCream
    else
      Sender.Canvas.Brush.Color := clWindow;
  end;
end;

procedure TfrmGN_SelectedPatients.loadFromList(aList:TList);
var
  p: Pointer;
  aPtData:TPatientData;
  sLocation:String;
begin
  for p in aList do
    if p <> nil then
      begin
        aPtData := TPatientData(p);
        sLocation := '???';
        addPatientData(aPtData,sLocation);
      end;
end;

procedure TfrmGN_SelectedPatients.SetFontSize(NewFontSize: Integer);
var
  i: Integer;
begin
  Font.Size := NewFontSize;
  lblInfo.Font.Size := NewFontSize;

  adjustBtn(bbRemove);
  adjustBtn(bbRemoveAll);

  for i := 0 to lvGnPtList.Columns.Count - 1 do
    begin
      lvGnPtList.Columns[i].Width :=
        getMainFormTextWidth(lvGnPtList.Columns[i].Caption) + GAP*2;
    end;
  lvGnPtList.Repaint;
end;

end.
