unit fRTTIView;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants,
  System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, fBase508Form, VA508AccessibilityManager,
  Vcl.StdCtrls, Vcl.ComCtrls, Vcl.ExtCtrls, System.Actions,
  Vcl.ActnList, Vcl.Buttons, Vcl.ToolWin, System.ImageList, Vcl.ImgList;

type
  TfrmRTTI = class(TfrmBase508Form)
    pnlCanvas: TPanel;
    splDebug: TSplitter;
    tv: TTreeView;
    pnlTools: TPanel;
    sb: TStatusBar;
    pnlLeft: TPanel;
    Panel1: TPanel;
    pnlDetails: TPanel;
    edTarget: TEdit;
    ActionList1: TActionList;
    acAllTypes: TAction;
    mmDetails: TMemo;
    lblItem: TLabel;
    Label1: TLabel;
    ToolBar1: TToolBar;
    ToolButton10: TToolButton;
    ToolButton11: TToolButton;
    ToolButton12: TToolButton;
    ToolButton13: TToolButton;
    c: TImageList;
    acAlignLeft: TAction;
    acAlignRight: TAction;
    procedure tvChange(Sender: TObject; Node: TTreeNode);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure acAlignLeftExecute(Sender: TObject);
    procedure acAlignRightExecute(Sender: TObject);
    procedure acAllTypesExecute(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    indent, indentStep: String;
    _Ignore: Boolean;
    procedure LogClear;
    procedure Clear;
    procedure doTVChange(Node: TTreeNode);
    function findNodeObject(aNode: TTreeNode; anObj: TObject): Boolean;
    // procedure doTargetFind(aTarget: String);
    // procedure setTypeNode(aNode: TTreeNode; aName, aTypeName: String);
  public
    { Public declarations }
    procedure doGetTypes;
    procedure LogUpdate(aText: String);
    procedure AddObject(anObject: TObject; aName: String = '');
    procedure AddObjectList(aList: TStringList; aName: String;
      addNils: Boolean = False);
    procedure WriteObjectRTTI(Node: TTreeNode);
  end;

procedure ShowRTTI(anObject: TObject);
procedure CreateRTTI;

implementation

uses
  System.RTTI, uGN_Utils; //uFormUtils;

const
  CRLF = #13#10;

{$R *.dfm}

var
  frmRTTI: TfrmRTTI;

procedure CreateRTTI;
begin
  if not assigned(frmRTTI) then
    Application.CreateForm(TfrmRTTI, frmRTTI);
end;

procedure ShowRTTI(anObject: TObject);
begin
  if not assigned(frmRTTI) then
    CreateRTTI;
  if assigned(frmRTTI) then
  begin
    frmRTTI.AddObject(anObject, anObject.ClassName);
    frmRTTI.Show;
  end;
end;

/// /////////////////////////////////////////////////////////////////////////////

procedure TfrmRTTI.tvChange(Sender: TObject; Node: TTreeNode);
begin
  if _Ignore then
    exit;
  doTVChange(Node);
end;

function TfrmRTTI.findNodeObject(aNode: TTreeNode; anObj: TObject): Boolean;
var
  nn: TTreeNode;
begin
  Result := False;
  nn := aNode.Parent;
  if assigned(nn) and (nn.Data = Pointer(anObj)) then
  begin
    Result := True;
    exit;
  end;

  nn := aNode.getFirstChild;
  while nn <> nil do
  begin
    Result := nn.Data = Pointer(anObj);
    if Result then
      exit
    else
      nn := nn.GetNext;
  end;
end;

procedure TfrmRTTI.WriteObjectRTTI(Node: TTreeNode);
var
  aLine, aClassName: String;
  anObj, _Obj: TObject;
  aContext: TRttiContext;
  aType: TRttiType;
  aField: TRttiField;

const
  fmtNameValue = '%-25.25s %s';
  fmtNameClassValue = '%-25.25s (%-25.25s) %s';

begin
  anObj := TObject(Node.Data);

  if anObj = nil then
    exit;
  if anObj.ClassType = nil then
    exit;

  if anObj is TStringList then
  begin
    for aLine in TStringList(anObj) do
      mmDetails.Lines.Add(indent + aLine);
    exit;
  end;

  try
    aType := aContext.GetType(anObj.ClassType);
    for aField in aType.GetFields do
      if aField.FieldType.IsInstance then
      begin
        _Obj := nil;
        try
          _Obj := aField.GetValue(anObj).AsObject;
          if assigned(_Obj) then
          begin
            aClassName := '(' + _Obj.ClassName + ')';
            if not findNodeObject(Node, _Obj) then
              tv.Items.AddChildObject(Node, aField.Name + aClassName, _Obj);
          end
          else
            aClassName := '';
          LogUpdate(indent + Format(fmtNameValue, [aField.Name, aClassName]));
        except
          on E: Exception do
            if assigned(_Obj) then
              LogUpdate(_Obj.ClassName + CRLF + E.Message);
        end;
      end
      else
        LogUpdate(indent + Format(fmtNameValue,
          [aField.Name, aField.GetValue(anObj).ToString]));

  except
    on E: Exception do
    begin
      LogUpdate('ClassName <' + anObj.ClassType.ClassName + '>');
      LogUpdate(E.Message);
    end;
  end;
end;

procedure TfrmRTTI.doTVChange(Node: TTreeNode);
begin
  inherited;
  mmDetails.Clear;
  lblItem.Caption := Node.Text;
  WriteObjectRTTI(Node);
end;

procedure TfrmRTTI.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  inherited;
  Hide;
  Action := caNone;
end;

procedure TfrmRTTI.FormCreate(Sender: TObject);
begin
  inherited;
  indent := '';
  indentStep := '  ';
end;

procedure TfrmRTTI.FormDestroy(Sender: TObject);
begin
  Clear;
  inherited;

end;

procedure TfrmRTTI.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  inherited;
  if Key = VK_ESCAPE then
  begin
    Key := 0;
    Hide;
  end;
end;

procedure TfrmRTTI.LogUpdate(aText: string);
begin
  mmDetails.Lines.Add(aText);
end;

procedure TfrmRTTI.LogClear;
begin
  mmDetails.Lines.Clear;
end;

procedure TfrmRTTI.Clear;
begin
  LogClear;
  tv.Items.Clear;
end;

procedure TfrmRTTI.acAlignLeftExecute(Sender: TObject);
begin
  inherited;
  AlignForm(self, alLeft);
end;

procedure TfrmRTTI.acAlignRightExecute(Sender: TObject);
begin
  inherited;
  AlignForm(self, alRight);
end;

procedure TfrmRTTI.acAllTypesExecute(Sender: TObject);
begin
  inherited;
  doGetTypes;
end;

procedure TfrmRTTI.AddObject(anObject: TObject; aName: String = '');
var
  sName: String;
  b: Boolean;

  function FindObject(anObject: TObject): TTreeNode;
  var
    tn: TTreeNode;
  begin
    Result := nil;
    for tn in tv.Items do
    begin
      if tn.Data = Pointer(anObject) then
      begin
        Result := tn;
        break
      end;
    end;

    if Result <> nil then
      // tv.Select(Result);
      tv.Selected := Result;
  end;

begin
  if not assigned(anObject) then
    exit;

  if FindObject(anObject) <> nil then
    exit;

  if trim(aName) = '' then
    sName := anObject.ClassName
  else
    sName := aName;
  tv.Enabled := False;
  b := _Ignore;
  _Ignore := True;
  tv.Items.AddObject(nil, sName, anObject);
  _Ignore := b;
  tv.Enabled := True;
end;

procedure TfrmRTTI.AddObjectList(aList: TStringList; aName: String;
  addNils: Boolean = False);
var
  s: String;
  i: Integer;
  obj: TObject;
begin
  for i := 0 to aList.Count - 1 do
  begin
    s := aList[i];
    obj := TObject(aList.Objects[i]);
    if (obj <> nil) or addNils then
      AddObject(obj, s);
  end;
end;

/// /////////////////////////////////////////////////////////////////////////////
(*
  procedure TfrmRTTI.doTargetFind(aTarget: String);
  var
  aContext: TRttiContext;
  aType: TRttiType;
  aProperty: TRttiProperty;
  const
  fmtProperty = ' %-25.25s - %s';
  begin
  LogClear;
  LogUpdate(aTarget + CRLF);
  aType := aContext.FindType(aTarget);
  if not assigned(aType) then
  exit;
  LogUpdate(aType.QualifiedName + CRLF);

  for aProperty in aType.GetProperties do
  begin
  LogUpdate(Format(fmtProperty, [aProperty.Name,
  aProperty.PropertyType.Name]));
  setTypeNode(nil, aProperty.Name, aProperty.PropertyType.Name);
  end;
  end;
*)

procedure TfrmRTTI.doGetTypes;
var
  aContext: TRttiContext;
  theTypes: TArray<TRttiType>;
  aType: TRttiType;
  tn: TTreeNode;

  procedure addNode(aName: String);
  begin
    tn := tv.Items.AddChild(nil, aName);
  end;

begin
  tv.Items.Clear;
  try
    theTypes := aContext.GetTypes;
    for aType in theTypes do
      addNode(aType.QualifiedName);
  finally

  end;
end;

(*
  procedure TfrmRTTI.setTypeNode(aNode: TTreeNode; aName, aTypeName: String);
  var
  aContext: TRttiContext;
  aProperty: TRttiProperty;
  aType: TRttiType;
  tn: TTreeNode;
  begin
  aType := aContext.FindType(aTypeName);
  if not assigned(aType) then
  exit;
  tn := tv.Items.AddChild(aNode, aName + ' (' + aTypeName + ')');
  for aProperty in aType.GetProperties do
  setTypeNode(tn, aProperty.Name, aProperty.PropertyType.Name);

  end;
*)
initialization

end.
