unit oGN_PtData;

interface
uses
  System.Classes,
  uTIU, uPCE, uCore;

type
  TPatientData = Class(TObject)
  private
    FOKForSign: Boolean;
    FPatient: TPatient;
    FAdditionalSigner: boolean;

  public
    PtNote: TEditNoteRec;
    PCEData: TPCEData;
    property OKForSign: boolean read FOKForSign  write FOKForSign;
    property Patient: TPatient read FPatient write FPatient;
    property AdditionalSigner: boolean read FAdditionalSigner write FAdditionalSigner;

    constructor Create;
    procedure Clear;
    procedure ErasePCEData;
    function toText: String;
    function PCESummary: String;
//    function VitalsByNoteIEN: String;
//    function VitalsByEncounterDateTime: String;
  end;

var
  GNPtList: TList;

procedure EraseAllPCEData;
function ClonePatientData(aData: TPatientData): TPatientData;

implementation
uses
  System.SysUtils, rTIU, rVitals;

procedure EraseAllPCEData;
var
  i: Integer;
  pd: TPatientData;
begin
  for i := 0 to GNPtList.Count - 1 do
  begin
    pd := TPatientData(GNPtList.Items[i]);
    pd.ErasePCEData;
  end;
end;

function ClonePatientData(aData: TPatientData): TPatientData;
begin
  Result := nil;
  if not assigned(aData) then
    exit;

  Result := TPatientData.Create;
  aData.PCEData.CopyPCEData(Result.PCEData);
  Result.Patient.DFN := aData.Patient.DFN; // will recreate patient;
  Result.OKForSign := aData.OKForSign;
  Result.AdditionalSigner := aData.AdditionalSigner;

  with Result.PtNote do
  begin
    NoteIEN      := aData.PtNote.NoteIEN;
    DocType      := aData.PtNote.DocType;
    Title        := aData.PtNote.Title;
    TitleName    := aData.PtNote.TitleName;
    DateTime     := aData.PtNote.DateTime;
    Author       := aData.PtNote.Author;
    AuthorName   := aData.PtNote.AuthorName;
    Cosigner     := aData.PtNote.Cosigner;
    CosignerName := aData.PtNote.CosignerName;
    Subject      := aData.PtNote.Subject;
    Location     := aData.PtNote.Location;
    LocationName := aData.PtNote.LocationName;
    PkgIEN       := aData.PtNote.PkgIEN;
    PkgPtr       := aData.PtNote.PkgPtr;
    PkgRef       := aData.PtNote.PkgRef;
    NeedCPT      := aData.PtNote.NeedCPT;
    Addend       := aData.PtNote.Addend;
    Lines        := TStringList.Create;
    Lines.Assign(aData.PtNote.Lines);
    GenLines     := TStringList.Create;
    GenLines.Assign(aData.PtNote.GenLines);
    PtLines      := TStringList.Create;
    PtLines.Assign(aData.PtNote.PtLines);
    ErrTxt       := aData.PtNote.ErrTxt;
    SignStatus   := aData.PtNote.SignStatus;
  end;
end;

{ TPatientData }
procedure TPatientData.Clear;
begin
  with PtNote do
  begin
    NoteIEN      := 0;
    DocType      := 0;
    Title        := 0;
    TitleName    := '';
    DateTime     := 0;
    Author       := 0;
    AuthorName   := '';
    Cosigner     := 0;
    CosignerName := '';
    Subject      := '';
    Location     := 0;
    LocationName := '';
    PkgIEN       := 0;
    PkgPtr       := '';
    PkgRef       := '';
    NeedCPT      := False;
    Addend       := 0;
    Lines.Clear;
    GenLines.Clear;
    PtLines.Clear;
    ErrTxt       := '';
    SignStatus   := SS_NEW;
  end;
  FPatient.Free;
  PCEData.Free;
  FAdditionalSigner := False;
end;

constructor TPatientData.Create;
begin
  FOKForSign := False;
  FPatient := TPatient.Create;
  PCEData := TPCEData.Create;
  AdditionalSigner := False;
  with PtNote do
  begin
    NoteIEN      := 0;
    DocType      := 0;
    Title        := 0;
    TitleName    := '';
    DateTime     := 0;
    Author       := 0;
    AuthorName   := '';
    Cosigner     := 0;
    CosignerName := '';
    Subject      := '';
    Location     := 0;
    LocationName := '';
    PkgIEN       := 0;
    PkgPtr       := '';
    PkgRef       := '';
    NeedCPT      := False;
    Addend       := 0;
    Lines        := TStringList.Create;
    GenLines     := TStringList.Create;
    PtLines      := TStringList.Create;
    ErrTxt       := '';
    SignStatus   := SS_NEW;
  end;
end;

procedure TPatientData.ErasePCEData;
begin
  if PCEData.HasData then
  begin
    PCEData.setFlags(True,True);
    PCEData.Save(PtNote.NoteIEN, Patient.DFN, TRUE);
  end;
end;

function TPatientData.toText: String;
var
  s: String;
  sl: TStringList;

  function SafeString(aString: String): String;
  begin
    Result := StringReplace(aString, #0, '#0', [rfReplaceAll]);
  end;

  function getPCEItemString(aName: String; aList: TList): String;
  var
    anItem: TPCEItem;
    p: Pointer;
  begin
    Result := aName + #13#10;
    for p in aList do
      begin
        anItem := TPCEItem(p);
        if not assigned(anItem) then
          Continue;
        Result := Result + '        ' + anItem.ItemStr + #13#10;
        if anItem.Comment <> '' then
          Result := Result + '        ' + anItem.Comment + #13#10;
        Result := Result + '        ' + anItem.Status + #13#10;
      end;
  end;

begin
  Result :=          'PatientData ---------- begin' + #13#10;
  Result := Result + '  OKForSign         ' + BoolToStr(fOKForSign) + #13#10;
  Result := Result + '  Patient           ' + Patient.Name + '(' + Patient.DFN + ')' + #13#10;
  Result := Result + '  AdditionalSigner  ' + BoolToStr(FAdditionalSigner) + #13#10;
  Result := Result + '  Inpatient         ' + BoolToStr(Patient.Inpatient) + #13#10;
  Result := Result + '  PtNote (record) --------- begin' + #13#10;
  Result := Result + '    IEN:            ' + IntToStr(PtNote.NoteIEN) + #13#10;
  Result := Result + '    Title:          ' + PtNote.TitleName + #13#10;
  Result := Result + '    Subject:        ' + PtNote.Subject + #13#10;
  Result := Result + '    Location:       ' + PtNote.LocationName + #13#10;
  Result := Result + '    Date Time (FM): ' + FloatToStr(PtNote.DateTime) + #13#10;
  Result := Result + '    Author:         ' + PtNote.AuthorName + #13#10;
  Result := Result + '    Cosigner:       ' + PtNote.CosignerName + #13#10;
  Result := Result + '    SignStatus:     ' + IntToStr(PtNote.SignStatus) + #13#10;
  Result := Result + '    Visit Str (RPC):' + VisitStrForNote(PtNote.NoteIEN) + #13#10;
  Result := Result + '    Text: ' + #13#10;
  if assigned(PtNote.Lines) then
    begin
      for s in PtNote.Lines do
        Result := Result + '        ' + s + #13#10;
    end
  else
    Result := Result + '        not assigned yet'+ #13#10;
  Result := Result + '  PtNote (Record) --------- end' + #13#10;
  Result := Result + '  PCEData ---------- begin' + #13#10;
  Result := Result + '    Updated:         ' + PCEData.Updated.ToString + #13#10;
  Result := Result + '    Visit String:    ' + SafeString(PCEData.VisitString) + #13#10;
  Result := Result + '    Patient Name     ' + PCEData.PatientName + #13#10;
  Result := Result + '    Note IEN:        ' + IntToStr(PCEData.NoteIEN) + #13#10;
  Result := Result + '    Note Date/Time:  ' + FloatToStr(PCEData.NoteDateTime) + #13#10;
  Result := Result + '    Enc Date/Time:   ' + FloatToStr(PCEData.DateTime) + #13#10;
//  Providers
  Result := Result + '    VisitType:        ' + PCEData.StrVisitType + #13#10;
  Result := Result + '    Data Lists ---------- begin' + #13#10;
  Result := Result + getPCEItemString('      Diagnoses',PCEData.Diagnoses);
  Result := Result + getPCEItemString('      Procedures',PCEData.Procedures);
  Result := Result + getPCEItemString('      Immunization',PCEData.Immunizations);
  Result := Result + getPCEItemString('      SkinTests',PCEData.SkinTests);
  Result := Result + getPCEItemString('      Exams',PCEData.Exams);
  Result := Result + getPCEItemString('      PatientEds',PCEData.PatientEds);
  Result := Result + getPCEItemString('      HealthFactors',PCEData.HealthFactors);
  Result := Result + getPCEItemString('      GenFindings',PCEData.GenFindings);

  sl := PCEData.getPCEDataList;
  try
    for s in sl do
      Result := Result + '        ' + SafeString(s) + #13#10;
  finally
    sl.Free;
  end;

  Result := Result + '    Data Lists ----------- end' + #13#10;
  Result := Result + '  PCEData ---------- end' + #13#10;
  Result := Result + 'PatientData ---------- end' + #13#10;
end;

//Procedure GetVitalFromNoteIEN(Dest: TStrings; const PatientID: string; NoteIEN: Integer);  //*DFN*
(* Currently CPRS does not inlude Vitals in Summary report.
It might be different in a future
Commenting out code and keeping it just "in case"

function TPatientData.VitalsByNoteIEN: String;
var
  SL: TSTrings;
begin
  Result := '';
EXIT;
  if assigned(PCEData) then
    begin
      SL := TSTringList.Create;
      try
        GetVitalFromNoteIEN(SL, Patient.DFN, PCEData.NoteIEN);
        Result := SL.Text;
      finally
        SL.Free;
      end;
    end;
end;

//Procedure GetVitalsFromEncDateTime(Dest: TStrings; const PatientID: string; DateTime: TFMDateTime);  //*DFN*

function TPatientData.VitalsByEncounterDateTime: String;
var
  SL: TSTrings;
begin
  Result := '';
EXIT;
  if assigned(PCEData) then
    begin
      if PCEData.DateTime = 0.0 then
        exit;
      SL := TSTringList.Create;
      try
        GetVitalsFromEncDateTime(SL, Patient.DFN, PCEData.DateTime);
        Result := SL.Text;
      finally
        SL.Free;
      end;
    end;
end;
*)
function TPatientData.PCESummary;
begin
  Result := PCEDataToString(PCEData);

  if Result = '' then
    Result := 'No Data found';
end;

end.
