unit rGN_Core;

interface

uses SysUtils, Classes, Forms, ORNet, ORFn, ORClasses, uCore, rCore, oGN_PtData;

function getExternalName(anID,aFile: String):String;
function GetPatientData(APatientDFN: String): TPatientData;

function IsAuthUser: boolean;
function IsFullSSN(x: string): Boolean;
function IsLast5(x: string): Boolean;
function IsMaxICD10(Term: string): string;
function IsValidLocation(ALocation: integer): boolean;

procedure LoadDemographics(Dest: TStrings; const PtDFN: string);

procedure rpcGetRangeForEncs(var StartDays, StopDays: integer; DefaultParams: Boolean);
procedure rpcGetEncFutureDays(var FutureDays: string);

function setRPLPtList(aDest: TStrings; RPLJobNumber: string;
  const StartFrom: string; Direction: Integer): Integer;
function setSubSetOfPatients(aDest: TStrings; const StartFrom: string;
  Direction: Integer): Integer;


implementation

var
  uPtListDfltSort: string = '';  // Current user's patient selection list default sort order.

function getExternalName(anID,aFile: String):String;
begin
  try
    CallVistA('ORWU EXTNAME',[anID,aFile], Result,'');
  except
    Result := '';
  end;
end;

function IsLast5(x: string): Boolean;
var
  i: Integer;
begin
  Result := false;
  if not((Length(x) = 4) or (Length(x) = 5)) then
    Exit;
  if Length(x) = 5 then
  begin
    if not SysUtils.CharInSet(x[1],['A' .. 'Z', 'a' .. 'z']) then
      Exit;
    x := Copy(x, 2, 4);
  end;
  for i := 1 to 4 do
    if not SysUtils.CharInSet(x[i],['0' .. '9']) then
      Exit;
  Result := true;
end;

function IsFullSSN(x: string): Boolean;
var
  i: integer;
begin
  Result := False;
  if (Length(x) < 9) or (Length(x) > 12) then exit;
  case Length(x) of
    9:  for i := 1 to 9 do if not SysUtils.CharInSet(x[i],['0'..'9']) then Exit;
   10:  begin
          for i := 1 to 9 do if not SysUtils.CharInSet(x[i], ['0'..'9']) then Exit;
          if (Uppercase(x[10]) <> 'P') then Exit;
        end;
   11:  begin
          if (x[4] <> '-') or (x[7] <> '-') then Exit;
          x := Copy(x,1,3) + Copy(x,5,2) + Copy(x,8,4);
          for i := 1 to 9 do if not SysUtils.CharInSet(x[i], ['0'..'9']) then Exit;
        end;
   12:  begin
          if (x[4] <> '-') or (x[7] <> '-') then Exit;
          x := Copy(x,1,3) + Copy(x,5,2) + Copy(x,8,5);
          for i := 1 to 9 do if not SysUtils.CharInSet(x[i], ['0'..'9']) then Exit;
          if UpperCase(x[10]) <> 'P' then Exit;
        end;
  end;
  Result := True;
end;


function setRPLPtList(aDest: TStrings; RPLJobNumber: string;
  const StartFrom: string; Direction: Integer): Integer;
{ returns a pointer to a list of patients (for use in a long list box) -  The return value is
  a pointer to RPCBrokerV.Results, so the data must be used BEFORE the next broker call! }
begin
  CallVistA('ORQPT READ RPL', [RPLJobNumber, StartFrom, Direction], aDest);
  MixedCaseList(aDest);
  Result := aDest.Count;
end;
(*
function SubSetOfPatients(const StartFrom: string; Direction: Integer): TStrings;
{ returns a pointer to a list of patients (for use in a long list box) -  The return value is
  a pointer to RPCBrokerV.Results, so the data must be used BEFORE the next broker call! }
begin
  CallV('ORWPT LIST ALL', [StartFrom, Direction]);
  MixedCaseList(RPCBrokerV.Results);
  Result := RPCBrokerV.Results;
end;
*)
function setSubSetOfPatients(aDest: TStrings; const StartFrom: string;
  Direction: Integer): Integer;
{ returns a pointer to a list of patients (for use in a long list box) -  The return value is
  a pointer to RPCBrokerV.Results, so the data must be used BEFORE the next broker call! }
begin
  CallVistA('ORWPT LIST ALL', [StartFrom, Direction], aDest);
  MixedCaseList(aDest);
  Result := aDest.Count;
end;
procedure LoadDemographics(Dest: TStrings; const PtDFN: string);
begin
  CallV('ORWPT PTINQ', [PtDFN]);
  Dest.Assign(RPCBrokerV.Results);
end;

function IsValidLocation(ALocation: integer): boolean;
begin
  Result := (SCallV('ORWGN GNLOC',[ALocation])= '1' );
end;

function IsAuthUser: boolean;
begin
  Result := ( SCAllV('ORWGN AUTHUSR',[NIL]) = '1');
end;

function IsMaxICD10(Term: string): string;
begin
  // TDP - 09/11/2012 - Returns whether search term frequency is greater
  //       than Max allowed frequency and if it is, includes frequency count.
  Result := SCallV('ORWGN MAXFRQ',[Term])
end;

procedure rpcGetRangeForEncs(var StartDays, StopDays: integer; DefaultParams: Boolean);
var
  Start, Stop, Values: string;
begin
  if DefaultParams then
    CallVistA('ORWTPD1 GETEFDAT', [nil], Values)
  else
    CallVistA('ORWTPD1 GETEDATS', [nil], Values);
  Start := Piece(Values, '^', 1);
  Stop  := Piece(Values, '^', 2);
  StartDays := StrToIntDef(Start, 0);
  StopDays  := StrToIntDef(Stop, 0);
end;

procedure rpcGetEncFutureDays(var FutureDays: string);
begin
  CallVistA('ORWTPD1 GETEAFL', [nil], FutureDays);
end;

function GetPatientData(APatientDFN: String): TPatientData;
var
  i: integer;
begin
  Result := nil;
  for i := 0 to GNPtList.Count - 1 do
  begin
    if ( TPatientData(GNPtList.Items[i]).Patient.DFN = APatientDFN ) then
      begin
        Result := TPatientData(GNPtList.Items[i]);
        break;   // assuming there are no duplicates
      end;
  end;
end;

end.
