unit uGN_Utils;

interface

uses
  ORFn, rGN_Core, Windows, Controls, ExtCtrls, Buttons, ComCtrls, uCore, uTIU,
  Forms, Classes, System.SysUtils, StdCtrls, System.UITypes;

type
  TViewMode = (vmParentedForm, vmDialog, vmConfirmationDlg);

procedure MenuItemMoveRight(aMenu: hMenu; aCommand: Cardinal);

procedure setFormParented(aForm: TForm; aParent: TWinControl;
  anAlign: TAlign = alClient);

function HintString(aHint: String; aLimit: Integer = 80): String;
function LongDescriptionByCode(aCode, aDefault: String; aDate: TFMDateTime;
  AI10Active: Boolean): String;
function RemoveDuplicate(aTarget, aDelimeter: String): String;
function IsCorrectVersion(bDebug: Boolean): Boolean;
function getLongDescriptionByCode(aCode, aDefault: String): String;

function getMainFormTextHeight: Integer;
function getMainFormTextWidth(aText: String): Integer;
function getLabelTextWidth(aLabel: TLabel): Integer;

function getLabelHeight(aLabel:TLabel): Integer;
function getLabelWidth(aLabel:TLabel): Integer;
function getStaticTextHeight(aLabel: TStaticText): Integer;
function getStaticTextWidth(aLabel: TStaticText): Integer;

procedure adjustBitBtn(aBB: TBitBtn; ImageWidth: Integer = 16);
procedure adjustBtn(aBtn: TButton;IncludeParent:Boolean = false);

function EncounterToStr(anEncounter: TEncounter): String;
function EditNoteRecToStr(aRecord: TEditNoteRec): String;

procedure CleanPtList(aList: TList);

procedure adjustFormSize(aForm: TForm; X, Y: Integer);
procedure adjustFormPosition(aForm: TForm);

function LockForUpdate(CtrlToLock: TWinControl): Integer;
procedure UnlockForUpdate(Var ReSetVar: Integer; CtrlToLock: TWinControl);

function getTitleHeight(aForm: TForm): Integer;

procedure reHighlight(aRE: TRichEdit; StartChar, EndChar: Integer;
  HighLightColor: Integer);
procedure reHighlightDrop(aRE: TRichEdit);

procedure FocusWinControl(aControl: TWinControl);
//procedure setLogFontSize(aSize: Integer);

procedure GN_ResizeAnchoredFormToFont(aForm: TForm);

procedure AlignForm(aForm: TForm; AlignStyle: TAlign);

function RPCCheckList(aRPC: TSTrings; aDest: TSTrings;
  anOption: String = 'all'): Boolean;

procedure MissingRPCList;
function StartupLine:String;

procedure setLineSpace(anEdit:TRichEdit;aValue:Integer);

var
  _marginW: Integer;
  _marginH: Integer;

implementation

uses
  ORNet, rPCE, rMisc, VAUtils, ORSystem, Dialogs,
  fGN_RPCLog, uGN_RPCLog, uGN_Const,
  Math, WinApi.RichEdit, WinApi.Messages, uGlobalVar;

function LockForUpdate(CtrlToLock: TWinControl): Integer;
begin
  Result := CtrlToLock.Perform(EM_GETEVENTMASK, 0, 0);
  CtrlToLock.Perform(EM_SETEVENTMASK, 0, 0);
  CtrlToLock.Perform(WM_SETREDRAW, Ord(false), 0);
end;

procedure UnlockForUpdate(Var ReSetVar: Integer; CtrlToLock: TWinControl);
begin
  CtrlToLock.Perform(WM_SETREDRAW, Ord(true), 0);
  InvalidateRect(CtrlToLock.Handle, NIL, true);
  CtrlToLock.Perform(EM_SETEVENTMASK, 0, ReSetVar);
end;

procedure MenuItemMoveRight(aMenu: hMenu; aCommand: Cardinal);
var
  mii: TMenuItemInfo;
  Buffer: array [0 .. 79] of Char;

begin
  // GET Help Menu Item Info
  mii.cbSize := SizeOf(mii);
  mii.fMask := MIIM_TYPE;
  mii.dwTypeData := Buffer;
  mii.cch := SizeOf(Buffer);
  GetMenuItemInfo(aMenu, aCommand, false, mii);
  // SET Help Menu Item Info
  mii.fType := mii.fType or MFT_RIGHTJUSTIFY;
  SetMenuItemInfo(aMenu, aCommand, false, mii);
end;

function HintString(aHint: String; aLimit: Integer = 80): String;
var
  i: Integer;
  s, sHint: String;
  sl: TStringList;
begin
  // ICD-10. Removing duplicate data from Hint text
  if pos(' - ', aHint) > 0 then
    sHint := copy(aHint, pos(' - ', aHint) + 3, Length(aHint))
  else
    sHint := aHint;

  sHint := StringReplace(sHint, #9, ' ', [rfReplaceAll]);

  sl := TStringList.Create;
  sl.Delimiter := ' ';
  sl.DelimitedText := sHint;
  s := '';
  for i := 0 to sl.Count - 1 do
  begin
    if Length(s + sl[i]) > aLimit then
    begin
      Result := Result + s + #13#10;
      s := sl[i] + ' ';
    end
    else
      s := s + sl[i] + ' ';
  end;
  Result := Result + s;
end;

function FMDateToWindowsDate(aDate: String): TFMDateTime;
var
  year, month, day: Word;
begin
  year := StrToIntDef(copy(aDate, 1, 3), 0) + 1700;
  month := StrToIntDef(copy(aDate, 4, 2), 0);
  day := StrToIntDef(copy(aDate, 6, 2), 0);
  Result := EncodeDate(year, month, day);
end;

// ICD-10 Remediation. CodeCR:184
function getLongDescriptionByCode(aCode, aDefault: String): String;
var
  sl: TStringList;
  s: String;
begin
  Result := aDefault;
  sl := TStringList.Create;
  tCallV(sl, 'ORWPCE GET DX TEXT', ['', aCode]);
  if sl.Count > 0 then
  begin
    s := sl[0];
    FreeAndNil(sl);
    if (pos('-', s) = 1) or (trim(s) = '') then
      Result := aDefault
    else
      Result := copy(s, 4, Length(s));
  end
  else
    Result := aDefault;
end;

function LongDescriptionByCode(aCode, aDefault: String; aDate: TFMDateTime;
  AI10Active: Boolean): String;
var
  s: String;
  bExtend, bActive: Boolean;
  dDate: TFMDateTime;
  LexResults: TStringList;
begin
  if trim(aCode) = '' then
  begin
    Result := aDefault;
    Exit;
  end;

  LexResults := TStringList.Create;

  if pos('ICD-10', aDefault) > 0 then
  begin
    dDate := StrToFloat(GImplementationDate);
    bExtend := true;
    bActive := true;
  end
  else
  begin
    dDate := FMDateToWindowsDate(GImplementationDate) - 1;
    dDate := DateTimeToFMDateTime(dDate);
    bExtend := true;
    bActive := false;
  end;

  ListLexicon(LexResults, aCode, LX_ICD, dDate, bExtend, bActive);
  s := LexResults.Text;
  if (pos('-', s) = 1) or (trim(s) = '') then
    Result := aDefault
  else
    Result := Piece(s, U, 2) + ' (' + Piece(s, U, 3) + ' ' +
      Piece(s, U, 4) + ')';

  FreeAndNil(LexResults);
end;

function RemoveDuplicate(aTarget, aDelimeter: String): String;
begin
  if pos(aDelimeter, aTarget) > 0 then
    Result := copy(aTarget, pos(aDelimeter, aTarget) + Length(aDelimeter),
      Length(aTarget))
  else
    Result := aTarget;
end;

function IsCorrectVersion(bDebug: Boolean): Boolean;
var
  sApplication, sMessage, ClientVer, ServerVer, ServerReq: string;

  procedure LogInfoBox(aText, aCaption: String; aBtn: Integer);
  begin
    AddLogLine(aText, aCaption);
    InfoBox(aText, aCaption, aBtn);
  end;

begin
  Result := bDebug;

  if not IsAuthUser then
    InfoBox('You are not authorized to use Group Notes.', 'Group Notes', MB_OK or MB_ICONERROR)
  else
  begin
    sApplication := FileVersionValue(Application.ExeName,
      FILE_VER_FILEDESCRIPTION) + CRLF + CRLF + Char(VK_TAB) +
      FileVersionValue(Application.ExeName, FILE_VER_COMMENTS) + {CRLF +}
      Char(VK_TAB) + 'CRC: ' + IntToHex(CRCForFile(Application.ExeName), 8);
    ClientVer := ClientVersion(Application.ExeName);
    ServerReq := Piece(FileVersionValue(Application.ExeName,
      FILE_VER_INTERNALNAME), ' ', 1);
    ServerVer := ServerVersion('OR GN SET LOCATIONS', ClientVer);
    sMessage := 'Client:          ' + ClientVer + CRLF + 'Server:          ' +
      ServerVer + CRLF + 'Required Server: ' + ServerReq;
    AddLogLine(sMessage, 'Version Info');

    if (ServerVer = '0.0.0.0') then
      LogInfoBox
        ('Unable to determine current version of server, please make sure you installed latest patch OR*3*222.',
        'Group Notes', MB_OK + MB_ICONWARNING)
    else
    begin
      ClientVer := ClientVersion(Application.ExeName);
      ServerReq := Piece(FileVersionValue(Application.ExeName,
        FILE_VER_INTERNALNAME), ' ', 1);

      sMessage := Char(VK_TAB) + 'Client:' + Char(VK_TAB) + Char(VK_TAB) +
        ClientVer + CRLF + CRLF + Char(VK_TAB) + 'Server: ' + Char(VK_TAB) +
        Char(VK_TAB) + ServerVer + CRLF + Char(VK_TAB) + 'Required:' +
        Char(VK_TAB) + ServerReq;

      if (CompareVersion(ServerVer, ClientVer) > 0) then
      // Server newer than Client
        LogInfoBox(TX_CLIENT_MISMATCH + CRLF + CRLF + sMessage,
          TC_CLIERR, MB_OK + MB_ICONWARNING)
      else
      begin
        if (CompareVersion(ServerVer, ServerReq) <> 0) then
        begin
          sMessage := 'This is ' + Char(VK_TAB) + sApplication + CRLF + CRLF +
            Char(VK_TAB) + ServerReq + '  ' + TX_VER_REQ + CRLF +
            Char(VK_TAB) + ServerVer + '  ' + TX_VER3 + CRLF +
            TX_VER_NEW;
          LogInfoBox(sMessage, TC_VER, MB_OK + MB_ICONERROR);
        end
        else
          Result := true;
      end;
    end;
  end;
end;

procedure setFormParented(aForm: TForm; aParent: TWinControl;
  anAlign: TAlign = alClient);
begin
  if aForm.Parent <> aParent then
  begin
    aForm.BorderStyle := bsNone;
    aForm.Parent := aParent;
    aForm.Align := anAlign;
    if not aForm.Visible then
      aForm.Show;
  end;
end;

function getMainFormTextHeight: Integer;
begin
  Result := 12;
  if Assigned(Application.MainForm) then
    Result := Application.MainForm.Canvas.TextHeight('qwerty1234567890');
end;

function getMainFormTextWidth(aText: String): Integer;
begin
  Result := 0;
  if Assigned(Application.MainForm) then
    Result := Application.MainForm.Canvas.TextWidth(aText);
end;

procedure adjustBitBtn(aBB: TBitBtn; ImageWidth: Integer = 16);
var
  i: Integer;
begin
  if not Assigned(Application.MainForm) then
    Exit;
  aBB.Font.Size := Application.MainForm.Font.Size;
  i := getMainFormTextWidth(aBB.Caption + 'W') + 2 * _marginW + ImageWidth;
  aBB.Width := i;
end;

const
  fmtName = '%-22.22s';
  fmtNameValue = '%-22.22s: %s';
  fmtNameValueD = '%-22.22s: %d';
  fmtNameValueCode = '%-22.22s: %s (%d)';

function NamedBool(aName: String; aValue: Boolean): String;
begin
  if aValue then
    Result := format(fmtNameValue, [aName, 'TRUE'])
  else
    Result := format(fmtNameValue, [aName, 'False']);
  Result := Result + CRLF;
end;

function EncounterToStr(anEncounter: TEncounter): String;
begin
  Result := 'Encounter: Not assigned';
  if not Assigned(anEncounter) then
    Exit;
  with anEncounter do
  begin
    Result := '';
    Result := Result + NamedBool('Need Visit', NeedVisit);
    Result := Result + format(fmtNameValue,
      ['Date/Time: ', DateTimeToStr(DateTime)]) + CRLF;
    Result := Result + NamedBool('Inpatient', Inpatient);
    Result := Result + format(fmtNameValueCode, ['Location (ID)', LocationName,
      Location]) + CRLF;
    Result := Result + format(fmtNameValue,
      ['Location Text', LocationText]) + CRLF;
    Result := Result + format(fmtNameValueCode, ['Provider (ID)', ProviderName,
      Provider]) + CRLF;
    Result := Result + NamedBool('Standalone', Standalone);
    Result := Result + format(fmtNameValue, ['Visit Category: ', '']) +
      VisitCategory + CRLF;
    Result := Result + format(fmtNameValue,
      ['Visit String: ', VisitStr]) + CRLF;
  end;
end;

function EditNoteRecToStr(aRecord: TEditNoteRec): String;

  function ListToString(aTitle: String; aList: TSTrings): String;
  var
    s: String;
  begin
    if not Assigned(aList) then
    begin
      Result := format(fmtNameValue, [aTitle, 'not assigned']) + CRLF;
    end
    else
    begin
      Result := '';
      if aList.Count < 1 then
        Result := format(fmtNameValue, [aTitle, '']) + CRLF
      else
        for s in aList do
        begin
          if Result = '' then
            Result := format(fmtNameValue, [aTitle, s]) + CRLF
          else
            Result := Result + format(fmtNameValue, ['', s]) + CRLF;
        end;
    end;
  end;

begin
  with aRecord do
  begin
    // Result := 'Note Record-------------------------------------------begin' + CRLF;
    Result := '';
    Result := Result + format(fmtNameValueD, ['NoteIEN', NoteIEN]) + CRLF;
    Result := Result + format(fmtNameValueD, ['DocType', DocType]) + CRLF;
    Result := Result + format(fmtNameValueCode, ['Title (IEN)', TitleName,
      Title]) + CRLF;
    // 'DateTime: ' + TFMDateTime;
    Result := Result + format(fmtNameValueCode, ['Author  (IEN)', AuthorName,
      Author]) + CRLF;
    Result := Result + format(fmtNameValueCode, ['Cosigner (IEN)', CosignerName,
      Cosigner]) + CRLF;
    Result := Result + format(fmtNameValue, ['Subject', Subject]) + CRLF;
    Result := Result + format(fmtNameValueCode, ['Location (IEN)', LocationName,
      Location]) + CRLF;
    // VisitDate: TFMDateTime;
    Result := Result + format(fmtNameValueCode, ['Package Ptr/Ref (IEN)',
      PkgPtr + '/' + PkgRef, PkgIEN]) + CRLF;
    Result := Result + format(fmtNameValueCode,
      ['LastCosigner (IEN)', LastCosignerName, LastCosigner]) + CRLF;

    Result := Result + NamedBool('Need CPT', NeedCPT) + CRLF;
    {
      Addend: Integer;
      IDParent: integer;
      ClinProcSummCode: integer;
      ClinProcDateTime: TFMDateTime;
    }
    Result := Result + ListToString('Lines', Lines);
    Result := Result + ListToString('Common Lines', GenLines);
    Result := Result + ListToString('Pt. Lines', PtLines);

    if ErrTxt <> '' then
      Result := Result + 'Error text:' + ErrTxt + CRLF;

    Result := Result + CRLF;
    {
      PRF_IEN: integer;
      ActionIEN: string;
    }
  end;
end;

procedure CleanPtList(aList: TList);
begin
  if not Assigned(aList) then
    Exit;

  while aList.Count > 0 do
  begin
    if aList.Items[0] <> nil then
      try
        TPatient(aList.Items[0]).Free;
        aList.Delete(0);
      except
        raise;
      end;
  end;
end;


function GetMonitor(aForm:TForm): TMonitor;
var
  M: TMonitor;
  I: Integer;
begin
  Result := nil;
  M := Screen.MonitorFromWindow(aForm.Handle, mdNearest);
  for I := 0 to Screen.MonitorCount - 1 do
    if Screen.Monitors[I].Handle = M.Handle then
    begin
      Result := Screen.Monitors[I];
      Exit;
    end;
end;


procedure adjustFormSize(aForm: TForm; X, Y: Integer);
var
  R: TRect;
  Monitor: TMonitor;
begin
  Monitor := getMonitor(aForm);
  R := Monitor.WorkAreaRect;

  if X > (R.Right - R.Left) then
    X := (R.Right - R.Left);

  if Y > (R.Bottom - R.Top) then
    Y := (R.Bottom - R.Top);

  aForm.Width := round(X);
  aForm.height := round(Y);
end;

procedure adjustFormPosition(aForm: TForm);
var
  R: TRect;
  Monitor: TMonitor;
begin
  Monitor := getMonitor(aForm);
  R := Monitor.WorkAreaRect;
  if aForm.Left + aForm.Width > R.Right then
    aForm.Left := max(R.Left, (R.Right - aForm.Width));

  if aForm.Top + aForm.height > R.Bottom then
    aForm.Top := max(R.Top, R.Bottom - aForm.height);
end;

function getTitleHeight(aForm: TForm): Integer;
var
  P: TPoint;
  R: TRect;
begin
  P.X := 0;
  P.Y := 0;
  P := aForm.ClientToScreen(P);
  getWindowRect(aForm.Handle, R);
  Result := P.Y - R.Location.Y;
end;

procedure reHighlight(aRE: TRichEdit; StartChar, EndChar: Integer;
  HighLightColor: Integer);
var
  format: CHARFORMAT2;
begin
  aRE.SelStart := StartChar;
  aRE.SelLength := EndChar;
  // Set the background color
  FillChar(format, SizeOf(format), 0);
  format.cbSize := SizeOf(format);
  format.dwMask := CFM_BACKCOLOR;
  format.crBackColor := HighLightColor;
  aRE.Perform(EM_SETCHARFORMAT, SCF_SELECTION, Longint(@format));
end;

procedure reHighlightDrop(aRE: TRichEdit);
var
  format: CHARFORMAT2;
begin
  // Set the background color
  FillChar(format, SizeOf(format), 0);
  format.cbSize := SizeOf(format);
  format.dwMask := CFM_BACKCOLOR;
  format.dwEffects := CFE_AUTOBACKCOLOR;
  aRE.Perform(EM_SETCHARFORMAT, SCF_SELECTION, Longint(@format));
end;

procedure FocusWinControl(aControl: TWinControl);
begin
  if Assigned(aControl) and aControl.CanFocus then
    try
      aControl.SetFocus;
    except
      on E: Exception do
        ShowMessage('DEBUG: ' + aControl.Name + CRLF + E.Message);
    end;
end;

//procedure setLogFontSize(aSize: Integer);
//begin
//  if Assigned(frmRPCLog) then
//    frmRPCLog.setFontSize(aSize);
//end;

procedure GN_ResizeAnchoredFormToFont(aForm: TForm);
begin
  if EnableFormResizing then
    ResizeAnchoredFormToFont(aForm);
end;

procedure AlignForm(aForm: TForm; AlignStyle: TAlign);
var
  frm: TForm;
  _Monitor: TMonitor;
  iLog: Integer;
  R: TRect;
const
  RpcLogPart = 3;

begin
  with aForm do
  begin
    frm := Application.MainForm;
    _Monitor := Screen.MonitorFromPoint(Mouse.CursorPos);
    R := _Monitor.WorkareaRect;
    iLog := R.Width div RpcLogPart;
    frm.Top := R.Top;
    frm.height := R.Bottom - R.Top;
    Top := R.Top;
    height := R.Bottom - R.Top;
    Width := iLog;

    case AlignStyle of
    alNone:
      begin
        frm.Left := R.Left;
        frm.Width := R.Width;
        Left := R.Left + R.Width - iLog;
      end;
    alTop: ;
    alBottom: ;
    alLeft: begin
        frm.Left := R.Left + iLog;
        frm.Width := R.Width - iLog;
        Left := R.Left;
      end;
    alRight:
      begin
        frm.Left := R.Left;
        frm.Width := R.Width - iLog;
        Left := R.Left + frm.Width;
      end;
    alClient: ;
    alCustom: ;
    end;
  end;
end;

function RPCCheckList(aRPC: TSTrings; aDest: TSTrings;
  anOption: String = 'all'): Boolean;
var
  sl: TSTrings;
  s: String;
  i: Integer;
begin
  sl := TStringList.Create;
  try
    CallVistA('XWB ARE RPCS AVAILABLE', ['L', aRPC], sl);
    Result := aRPC.Count = sl.Count;
    aDest.Clear;
    if Result then
      for i := 0 to sl.Count - 1 do
      begin
        if (sl[i] = '1') or (aRPC[i] = ' ') or (pos('---', aRPC[i]) = 1) then
          s := ' '
        else
          s := '?';
        if (anOption = 'all') then
          aDest.Add(sl[i] + ' ' + s + ' ' + aRPC[i])
        else if (s = '?') then
          aDest.Add(aRPC[i]);
      end;
  finally
    sl.Free;
  end;
end;

procedure TestApplicationRPC(const slOut: TSTrings);
var
  sl: TStringList;
begin
  sl := TStringList.Create;

  sl.Add('DG CHK BS5 XREF ARRAY');
  sl.Add('DG SENSITIVE RECORD ACCESS');
  sl.Add('DG SENSITIVE RECORD BULLETIN');
  sl.Add('ORBSMART ISSMNOT');
  sl.Add('ORDEA CSVALUE');
  sl.Add('ORQPT CLINIC PATIENTS');
  sl.Add('ORQPT DEFAULT CLINIC DATE RANG');
  sl.Add('ORQPT DEFAULT LIST SORT');
  sl.Add('ORQPT DEFAULT LIST SOURCE');
  sl.Add('ORQPT MAKE RPL');
  sl.Add('ORQPT PROVIDER PATIENTS');
  sl.Add('ORQPT READ RPL');
  sl.Add('ORQPT SPECIALTIES');
  sl.Add('ORQPT SPECIALTY PATIENTS');
  sl.Add('ORQPT TEAM PATIENTS');
  sl.Add('ORQPT TEAMS');
  sl.Add('ORQPT WARDS');
  sl.Add('ORQQCN ADDCMT');
  sl.Add('ORQQCN ADMIN COMPLETE');
  sl.Add('ORQQCN ASSIGNABLE MED RESULTS');
  sl.Add('ORQQCN ATTACH MED RESULTS');
  sl.Add('ORQQCN CANEDIT');
  sl.Add('ORQQCN DEFAULT REQUEST REASON');
  sl.Add('ORQQCN DETAIL');
  sl.Add('ORQQCN DISCONTINUE');
  sl.Add('ORQQCN EDIT DEFAULT REASON');
  sl.Add('ORQQCN FIND CONSULT');
  sl.Add('ORQQCN FORWARD');
  sl.Add('ORQQCN GET CONSULT');
  sl.Add('ORQQCN GET MED RESULT DETAILS');
  sl.Add('ORQQCN GET ORDER NUMBER');
  sl.Add('ORQQCN GET PROC IEN');
  sl.Add('ORQQCN GET PROC SVCS');
  sl.Add('ORQQCN GET SERVICE IEN');
  sl.Add('ORQQCN GET USER AUTH');
  sl.Add('ORQQCN ISPROSVC');
  sl.Add('ORQQCN LIST');
  sl.Add('ORQQCN MED RESULTS');
  sl.Add('ORQQCN PRINT SF513');
  sl.Add('ORQQCN PROVDX');
  sl.Add('ORQQCN RECEIVE');
  sl.Add('ORQQCN REMOVABLE MED RESULTS');
  sl.Add('ORQQCN REMOVE MED RESULTS');
  sl.Add('ORQQCN SET ACT MENUS');
  sl.Add('ORQQCN SF513 WINDOWS PRINT');
  sl.Add('ORQQCN SHOW SF513');
  sl.Add('ORQQCN SIGFIND');
  sl.Add('ORQQCN STATUS');
  sl.Add('ORQQCN SVC W/SYNONYMS');
  sl.Add('ORQQCN SVCLIST');
  sl.Add('ORQQCN SVCTREE');
  sl.Add('ORQQCN UNRESOLVED');
  sl.Add('ORQQCN URGENCIES');
  sl.Add('ORQQCN2 GET CONTEXT');
  sl.Add('ORQQCN2 GET PREREQUISITE');
  sl.Add('ORQQCN2 SAVE CONTEXT');
  sl.Add('ORQQCN2 SCHEDULE CONSULT');
  sl.Add('ORUTL4 DLL');
  sl.Add('ORWCH LOADSIZ');
  sl.Add('ORWCIRN JLV LABEL');
  sl.Add('ORWDCN32 DEF');
  sl.Add('ORWDCN32 NEWDLG');
  sl.Add('ORWDCN32 ORDRMSG');
  sl.Add('ORWDCN32 PROCEDURES');
  sl.Add('ORWDXQ GETQLST');
  sl.Add('ORWGN AUTHUSR');
  sl.Add('ORWGN GNLOC');
  sl.Add('ORWGN IDTVALID');
  sl.Add('ORWGN MAXFRQ');
  // SL.Add('ORWLEX GETALTS'); -- commented out in rPCE
  sl.Add('ORWLEX GETI10DX');
  sl.Add('ORWORB GETDATA');
  sl.Add('ORWORB GETSORT');
  sl.Add('ORWORB UNSIG ORDERS FOLLOWUP');
  sl.Add('ORWPCE ACTIVE PROV');
  sl.Add('ORWPCE CPTMODS');
  sl.Add('ORWPCE CPTREQD');
  sl.Add('ORWPCE DELETE');
  sl.Add('ORWPCE GAFURL');
  sl.Add('ORWPCE GET DX TEXT');
  sl.Add('ORWPCE GET EXCLUDED');
  sl.Add('ORWPCE GET VISIT');
  sl.Add('ORWPCE GETSVC');
  sl.Add('ORWPCE I10IMPDT');
  sl.Add('ORWPCE ICDVER');
  sl.Add('ORWPCE NOTEVSTR');
  sl.Add('ORWPCE PCE4NOTE');
  sl.Add('ORWPCE SAVE');
  sl.Add('ORWPCE SCDIS');
  sl.Add('ORWPCE SCSEL');
  sl.Add('ORWPCE VISIT');
  sl.Add('ORWPCE4 LEX');
  sl.Add('ORWPT BYWARD');
  sl.Add('ORWPT CLINRNG');
  sl.Add('ORWPT DFLTSRC');
  sl.Add('ORWPT ENCTITL');
  sl.Add('ORWPT FULLSSN');
  sl.Add('ORWPT FULLSSN RPL');
  sl.Add('ORWPT ID INFO');
  sl.Add('ORWPT INPLOC');
  sl.Add('ORWPT LAST5');
  sl.Add('ORWPT LAST5 RPL');
  sl.Add('ORWPT LIST ALL');
  sl.Add('ORWPT PTINQ');
  sl.Add('ORWPT SELECT');
  sl.Add('ORWPT SHARE');
  sl.Add('ORWPT TOP');
  sl.Add('ORWPT1 PRCARE');
  sl.Add('ORWRP GET DEFAULT PRINTER');
  sl.Add('ORWTIU CANLINK');
  sl.Add('ORWTIU GET LISTBOX ITEM');
  sl.Add('ORWTIU GET SAVED CP FIELDS');
  sl.Add('ORWTIU GET TIU CONTEXT');
  sl.Add('ORWU CLINLOC');
  sl.Add('ORWU DT');
  sl.Add('ORWU EXTNAME');
  sl.Add('ORWU GBLREF');
  sl.Add('ORWU GENERIC');
  sl.Add('ORWU HASKEY');
  sl.Add('ORWU NEWPERS');
  sl.Add('ORWU NPHASKEY');
  sl.Add('ORWU OVERDL');
  sl.Add('ORWU PARAM');
  sl.Add('ORWU PATCH');
  sl.Add('ORWU USERINFO');
  sl.Add('ORWU VALIDSIG');
  sl.Add('ORWU VERSION');
  sl.Add('ORWU VERSRV');
  sl.Add('ORWU1 NAMECVT');
  sl.Add('TIU AUTHORIZATION');
  sl.Add('TIU GET ALERT INFO');
  sl.Add('TIU GET DEFAULT PROVIDER');
  sl.Add('TIU GET DOCUMENT PARAMETERS');
  sl.Add('TIU GET PERSONAL PREFERENCES');
  sl.Add('TIU GET PRINT NAME');
  sl.Add('TIU GET REQUEST');
  sl.Add('TIU GET SITE PARAMETERS');
  sl.Add('TIU HAS AUTHOR SIGNED?');
  sl.Add('TIU ID ATTACH ENTRY');
  sl.Add('TIU ID CAN ATTACH');
  sl.Add('TIU ID CAN RECEIVE');
  sl.Add('TIU ID DETACH ENTRY');
  sl.Add('TIU IS THIS A CLINPROC?');
  sl.Add('TIU IS THIS A CONSULT?');
  sl.Add('TIU ISPRF');
  sl.Add('TIU JUSTIFY DELETE?');
  sl.Add('TIU LOCK RECORD');
  sl.Add('TIU ONE VISIT NOTE?');
  sl.Add('TIU PERSONAL TITLE LIST');
  sl.Add('TIU PRINT RECORD');
  sl.Add('TIU REQUIRES COSIGNATURE');
  sl.Add('TIU SIGN RECORD');
  sl.Add('TIU USER INACTIVE?');
  sl.Add('TIU WAS THIS SAVED?');
  sl.Add('TIU WHICH SIGNATURE ACTION');

  try
    RPCCheckList(sl, slOut, 'Missing');
    if slOut.Count > 0 then
      InfoBox('The following RPC might be missing:'+ CRLF+CRLF+slOut.Text,
        'RPC Check', MB_OK + MB_ICONWARNING);
  finally
    sl.Free;
  end;

end;

procedure MissingRPCList;
var
  sl: TStringList;
begin
  sl := TStringList.Create;
  TestApplicationRPC(sl);
  RPCLogInit;
  AddLogLine(sl.Text, 'Missing RPC List');
  sl.Free;
end;

function StartupLine: String;
var
  i: Integer;
begin
  With TStringList.Create do
  begin
    for i := 0 to ParamCount do
      Add(System.paramStr(i));
    Result := Text;
    Free;
  end;
end;

procedure adjustBtn(aBtn: TButton; IncludeParent: Boolean = false);
var
  i: Integer;
begin
  aBtn.Width := getMainFormTextWidth(aBtn.Caption) + GAP + GAP;
  if aBtn.AlignWithMargins then
    aBtn.Width := aBtn.Width + aBtn.Margins.Left + aBtn.Margins.Right;

  aBtn.height := getMainFormTextHeight + GAP;
//  if aBtn.AlignWithMargins then
//    aBtn.height := aBtn.height + aBtn.Margins.Top + aBtn.Margins.Bottom;
  if IncludeParent and (aBtn.Parent is TPanel) then
  begin
    i := getMainFormTextHeight + GAP * 3;
    if TPanel(aBtn.Parent).height < i then
    begin
      TPanel(aBtn.Parent).height := i;
      if TPanel(aBtn.Parent).AlignWithMargins then
        TPanel(aBtn.Parent).height := TPanel(aBtn.Parent).height +
          TPanel(aBtn.Parent).Margins.Top + TPanel(aBtn.Parent).Margins.Bottom;
    end;
  end;
end;

function getLabelHeight(aLabel: TLabel): Integer;
begin
  Result := 0;
  if aLabel.Visible then
  begin
    Result := getMainFormTextHeight + GAP div 2;
    if aLabel.AlignWithMargins then
      Result := Result + aLabel.Margins.Top + aLabel.Margins.Bottom;
  end;
end;

function getLabelWidth(aLabel: TLabel): Integer;
begin
  Result := 0;
  if aLabel.Visible then
  begin
    Result := getMainFormTextWidth(aLabel.Caption) + GAP;
    if aLabel.AlignWithMargins then
      Result := Result + aLabel.Margins.Left + aLabel.Margins.Right;
  end;
end;

function getStaticTextHeight(aLabel: TStaticText): Integer;
begin
  Result := 0;
  if aLabel.Visible then
  begin
    Result := getMainFormTextHeight + GAP div 2;
    if aLabel.AlignWithMargins then
      Result := Result + aLabel.Margins.Top + aLabel.Margins.Bottom;
  end;
end;

function getStaticTextWidth(aLabel: TStaticText): Integer;
begin
  Result := 0;
  if aLabel.Visible then
  begin
    Result := getMainFormTextWidth(aLabel.Caption) + GAP;
    if aLabel.AlignWithMargins then
      Result := Result + aLabel.Margins.Left + aLabel.Margins.Right;
  end;
end;

procedure setLineSpace(anEdit:TRichEdit;aValue:Integer);
var
  Para: TParaFormat2;
begin
  Para.cbSize := SizeOf(Para);
  Para.dwMask := PFM_LINESPACING;
  Para.bLineSpacingRule := 4;  // Use exact twips specified
  Para.dyLineSpacing := 120;   // Ridiculously small value
  SendMessage(anEdit.Handle, EM_SETPARAFORMAT, 0, LPARAM(@Para));
end;

function getLabelTextWidth(aLabel: TLabel): Integer;
begin
  Result := 0;
  if Assigned(aLabel) then
    Result := aLabel.Canvas.TextWidth(aLabel.Caption);
end;
initialization

_marginW := 6;
_marginH := 3;

end.
