////////////////////////////////////////////////////////////////////////////////
//Modifed: 9/18/98
//By: Robert Bott
//Location: ISL
//Description of Mod:
//  Changed function ValAndStoreVitals to return the string indicating the value
//   that failed.
////////////////////////////////////////////////////////////////////////////////

unit rVitals;

{$OPTIMIZATION OFF}                              // REMOVE AFTER UNIT IS DEBUGGED

interface

uses SysUtils, Classes, ORNet, ORFn;

procedure SaveVitals(VitalList: TStringList);
Procedure GetLastVital(Dest: TStrings; const PatientID: string);  //*DFN*
function VerifyVital(typ,rte,unt: String):boolean;
function ValAndStoreVitals(VitalList: TStringList):string;      //9/18/98
Procedure GetVitalFromNoteIEN(Dest: TStrings; const PatientID: string; NoteIEN: Integer);  //*DFN*
Procedure GetVitalsFromEncDateTime(Dest: TStrings; const PatientID: string; datetime: TFMDateTime);  //*DFN*
procedure LoadUserVitalPreferences;
procedure SaveUserVitalPreferences;
function GetVList(Loc: integer): TStringList;

var
  uVitalsMetric: boolean = FALSE;
  uVitCVPmmHg: boolean = FALSE;

implementation

var
  uOldVitalsMetric: boolean = FALSE;
  uOldVitCVPmmHg: boolean = FALSE;
  uVitalList: TStringList = nil;
  uLastVitalLoc: integer = -1;

procedure SaveVitals(VitalList: TStringList);
begin
  CallV('ORWVITALS SAVE', [VitalList]);
end;

procedure GetLastVital(Dest: TStrings; const PatientID: string);  //*DFN*
var
  IDString: String;
begin
  IDString := patientid;  //*DFN*
  CallV('ORQQVI VITALS', [IDString]);
  Dest.assign(RPCBrokerV.Results);
end;

function VerifyVital(typ,rte,unt: String):boolean;
begin
  result := False;
  CallV('ORQQVI2 VITALS RATE CHECK',[typ,rte,unt]);
  if RPCBrokerV.results[0] = '1' then
    result := True;
end;


////////////////////////////////////////////////////////////////////////////////
//Modifed: 9/18/98
//By: Robert Bott
//Location: ISL
//Description of Mod:
//  Changed function ValAndStoreVitals to return the string indicating the value
//   that failed.
////////////////////////////////////////////////////////////////////////////////
function ValAndStoreVitals(VitalList: TStringList): string;
var
  i: Integer;
  sl: TStrings;

  function vName(aType,aCode:String): String;
  begin
    Result := '';
    if aType = 'VIT' then
      begin
        if aCode = 'TMP' then
          result := 'Temperature'
        else if aCode = 'PU' then
          result := 'Pulse'
        else if aCode = 'RS' then
          result := 'Respiration'
        else if aCode = 'BP' then
          result := 'Blood Preasure'
        else if aCode = 'HT' then
          result := 'Height'
        else if aCode = 'WT' then
          result := 'Weight'
        else if aCode = 'PN' then
          result := 'Pain'
        end;
  end;

  function getMessage(aValue: String): String;
  var
    s, s1, s2, s5: String;
    i: Integer;
  begin
    result := '';
    if aValue <> '' then
    begin
      result := piece(aValue, U, 2) + ': ' + piece(aValue, U, 3);
      s := piece(aValue, U, 4);
      if s <> '' then
        result := result + CRLF + 'Value: ' + s
      else
      begin
        s := '';
        i := strToIntDef(piece(aValue, U, 1), 0) - 1;
        if (i >= 0) and (i < VitalList.Count) then
        begin
          s := VitalList[i];
          s1 := piece(s, U, 1);
          s2 := piece(s, U, 2);
          s5 := piece(s, U, 5);
          s := vName(s1, s2);
          if s <> '' then
            s := ' ' + s + '  ' + s5;
        end;

        result := result + CRLF + s;
      end;
    end;
  end;

begin
  // CallV('ORQQVI2 VITALS VAL & STORE',[VitalList]);
  // if RPCBrokerV.results[0] >= '0' then
  // result := 'True'  //stored ok
  // else
  // result := Piece(RPCBrokerV.results[1],U,2) + ': '+Piece(RPCBrokerV.results[1],U,3)+' Value: '+
  // Piece(RPCBrokerV.results[1],U,4);
  result := '';
  sl := TStringList.Create;
  try
    if CallVistA('ORQQVI2 VITALS VAL & STORE', [VitalList], sl) then
    begin
      if sl.Count > 0 then
      begin
        if (sl[0] <> '0') or (sl.Count > 1) then
          for i := 1 to sl.Count - 1 do
            result := result + getMessage(sl[i]) + CRLF;
      end;
    end
    else
      result := 'Error executing RPC <ORQQVI2 VITALS VAL & STORE>';
  finally
    sl.Free;
  end;
end;

Procedure GetVitalFromNoteIEN(Dest: TStrings; const PatientID: string; NoteIEN: Integer);  //*DFN*
var
  NoteIENStr,IDString: String;
begin
  IDString := patientid;  //*DFN*
  NoteIENStr := IntToStr(NoteIen);
  CallV('ORQQVI NOTEVIT', [IDString, NoteIENStr]);
  Dest.assign(RPCBrokerV.Results);
end;

Procedure GetVitalsFromEncDateTime(Dest: TStrings; const PatientID: string; DateTime: TFMDateTime);  //*DFN*
var
  EncDate,IDString: String;
begin
  IDString := patientid;  //*DFN*
  EncDate := FloatToStr(DateTime);
  CallV('ORQQVI VITALS', [IDString, EncDate]);
  Dest.assign(RPCBrokerV.Results);
end;

procedure LoadUserVitalPreferences;
var
  Tmp: string;
begin
  Tmp := sCallV('ORQQVI3 GETVPREF', []);
  uVitalsMetric := (Piece(Tmp,U,1) = '1');
  uVitCVPmmHg   := (Piece(Tmp,U,2) = '1');
  uOldVitalsMetric := uVitalsMetric;
  uOldVitCVPmmHg := uVitCVPmmHg;
end;

procedure SaveUserVitalPreferences;
var
  P1, P2: string;

begin
  // Don't same same value loaded, since saving is at user level,
  //                     and loading may be at a different level
  if(uOldVitalsMetric = uVitalsMetric) then P1 := '' else P1 := BOOLCHAR[uVitalsMetric];
  if(uOldVitCVPmmHg   = uVitCVPmmHg)   then P2 := '' else P2 := BOOLCHAR[uVitCVPmmHg];
  CallV('ORQQVI3 SETVPREF', [P1, P2]);
end;

function GetVList(Loc: integer): TStringList;
var
  GetList: boolean;

begin
  GetList := TRUE;
  if not assigned(uVitalList) then
    uVitalList := TStringList.Create
  else
  if Loc = uLastVitalLoc then
    GetList := FALSE;
  if(GetList) then
  begin
    uLastVitalLoc := Loc;
    CallV('ORQQVI3 GETVLIST', [Loc]);
    if(RPCBrokerV.Results.Count > 0) then
      RPCBrokerV.Results.Delete(0);
    uVitalList.assign(RPCBrokerV.Results);
  end;
  Result := uVitalList;
end;

initialization

finalization
  KillObj(@uVitalList);

end.
