unit fPtSelDemog;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ORCtrls, fBase508Form, VA508AccessibilityManager,
  System.Actions, Vcl.ActnList, Vcl.Menus;

type
  TfrmPtSelDemog = class(TfrmBase508Form)
    Memo: TCaptionMemo;
    sbMain: TScrollBox;
    pmDemographics: TPopupMenu;
    alDemographics: TActionList;
    acCVShow: TAction;
    ShowCVData1: TMenuItem;
    gpDemog: TGridPanel;
    lblPtName: TStaticText;
    lblSSN: TStaticText;
    lblPtSSN: TStaticText;
    lblDOB: TStaticText;
    lblPtDOB: TStaticText;
    stxSex: TStaticText;
    lblPtSex: TStaticText;
    lblPtVet: TStaticText;
    stxVeteran: TStaticText;
    stxSC: TStaticText;
    lblPtSC: TStaticText;
    lblLocation: TStaticText;
    lblPtLocation: TStaticText;
    lblRoomBed: TStaticText;
    lblPtRoomBed: TStaticText;
    lblCombatVet: TStaticText;
    procedure FormShow(Sender: TObject);
    procedure MemoEnter(Sender: TObject);
    procedure MemoKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure acCVShowExecute(Sender: TObject);
  private
    FLastDFN: string;
    fCVShow: Boolean;
    procedure setLastDFN(aDFN: String);
    procedure ClearIDInfo;
  public
    property CVShow: Boolean read fCVShow write fCVShow;
    property LastDFN: String read fLastDFN write setLastDFN;
    procedure ShowDemog(ItemID: string);
    procedure ToggleMemo;
    procedure SetFontSize(NewFontSize: Integer); override;
  end;


implementation

uses rCore, VA508AccessibilityRouter, uCombatVet, ORFn, uGN_Utils;

{$R *.DFM}

const
{ constants referencing the value of the tag property in components }
  UNKNOWN      = ''; // 'unknown'; // ...?';
  NO_PATIENT_SELECTED = 'No patient selected';

procedure TfrmPtSelDemog.ClearIDInfo;
{ clears controls with patient ID info }
begin
  FLastDFN := '';
  Memo.Clear;
  lblCombatVet.Caption := '';
  lblPtName.Caption := NO_PATIENT_SELECTED;

  lblPtSSN.Visible := False;
  lblPtDOB.Visible := False;
  lblPtSex.Visible := False;
  lblPtName.Visible := False;
  lblPtVet.Visible := False;
  lblPtSC.Visible := False;
  lblPtLocation.Visible := False;
  lblPtRoomBed.Visible := False;
  lblCombatVet.Visible := False;
end;

procedure TfrmPtSelDemog.ShowDemog(ItemID: string);
{ gets a record of patient indentifying information from the server and displays it }
var
  PtRec: TPtIDInfo;
  CV : TCombatVet;

  procedure setLabels(aName,aValue:TStaticText;aText:String;aHide:Boolean = false);
  begin
    if aText = '' then
      aValue.Caption := UNKNOWN
    else
      aValue.Caption := aText;

    if aHide then
      begin
        aName.Visible := aText <> '';
        aValue.Visible := aText <> '';
      end;
  end;

begin
//  if ItemID = FLastDFN then Exit;
  FLastDFN := ItemID;

  Memo.Clear;

  lblPtSSN.Visible := True;
  lblPtDOB.Visible := True;
  lblPtSex.Visible := True;
  lblPtName.Visible := True;
  lblPtVet.Visible := True;
  lblPtSC.Visible := True;
  lblPtLocation.Visible := True;
  lblPtRoomBed.Visible := True;
  lblCombatVet.Visible := True;

  PtRec := GetPtIDInfo(fLastDFN);
  with PtRec do
  begin
    Memo.Lines.Add('Name: '+ Name);
    Memo.Lines.Add(lblSSN.Caption + ' ' + SSN + '.');
    Memo.Lines.Add(lblDOB.Caption + ' ' + DOB + '.');
    if Sex <> '' then
      Memo.Lines.Add(Sex + '.');
    if Vet <> '' then
      Memo.Lines.Add(Vet + '.');
    if SCsts <> '' then
      Memo.Lines.Add(SCsts + '.');
    if Location <> '' then
      Memo.Lines.Add(lblLocation.Caption + ' ' + Location + '.');
    if RoomBed <> '' then
      Memo.Lines.Add(lblRoomBed.Caption + ' ' + RoomBed + '.');

    lblPtName.Caption     := Name;
    lblPtSSN.Caption      := SSN;
    lblPtDOB.Caption      := DOB;
    if trim(PtRec.Name) <> '' then
      lblPtSex.Caption    := Sex // {+ ', age ' + Age};
    else
      lblPtSex.Caption    := '';

    setLabels(stxSC,lblPtSC,SCSts,true);
    setLabels(stxVeteran,lblPtVet,Vet,true);
    lblPtLocation.Caption := Location;
    lblPtRoomBed.Caption  := RoomBed;
  end;

  if CVShow then
  begin
    CV := TCombatVet.Create(ItemID);
    if CV.IsEligible then
    begin
      lblCombatVet.Caption := 'CV ' + CV.ExpirationDate + ' ' + CV.OEF_OIF;
      Memo.Lines.Add(lblCombatVet.Caption);
    end
    else
      lblCombatVet.Caption := '';
    CV.Free;
  end;

  formResize(nil);

  if ScreenReaderSystemActive then
  begin
    sbMain.Visible := False;

    Memo.TabStop := True;
    Memo.SelStart := 0;
    GetScreenReader.Speak('Selected Patient Demographics');
    GetScreenReader.Speak(Memo.Text);
  end;

end;

procedure TfrmPtSelDemog.setLastDFN(aDFN: string);
begin
  if aDFN = FLastDFN then
    Exit;

  if aDFN = '' then
    ClearIDInfo
  else
    ShowDemog(aDFN);
end;

procedure TfrmPtSelDemog.ToggleMemo;
{ toggle Memo visibility }
begin
  if Memo.Visible then
  begin
    Memo.Hide;
  end
  else
  begin
    Memo.Show;
    Memo.BringToFront;
  end;
end;

procedure TfrmPtSelDemog.FormCreate(Sender: TObject);
begin
  inherited;
  AutoScroll := False;
  LastDFN := '';

  sbMain.Visible := not ScreenReaderSystemActive;
  memo.Visible := ScreenReaderSystemActive;
  if memo.Visible then
    memo.BringToFront;
end;

procedure TfrmPtSelDemog.FormShow(Sender: TObject);
begin
  inherited;
  lblCombatVet.Caption := '';
end;

procedure TfrmPtSelDemog.MemoEnter(Sender: TObject);
/// reads the Memo contents if the Screenreader is active
begin
  inherited;
  if ScreenReaderSystemActive then
  begin
    Memo.SelStart := 0;
    GetScreenReader.Speak('Selected Patient Demographics');
    GetScreenReader.Speak(Memo.Text);
  end;
end;

procedure TfrmPtSelDemog.MemoKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
/// notifies on position in the Memo if the Screenreader is active
begin
  inherited;
  if ScreenReaderSystemActive then
  begin
    if Memo.SelStart = Memo.GetTextLen then
      if ((Key = VK_DOWN) or (Key = VK_RIGHT)) then GetScreenReader.Speak('End of Data');
    if Memo.SelStart = 0 then
      if ((Key = VK_UP) or (Key = VK_LEFT)) then GetScreenReader.Speak('Start of Data')
  end;
end;

procedure TfrmPtSelDemog.SetFontSize(NewFontSize: Integer);
var
  i,iHeight: Integer;
begin
  inherited;
  Font.Size := NewFontSize;
  lblPtName.Font.Size := NewFontSize;

  gpDemog.ColumnCollection.Items[0].Value :=
    getMainFormTextWidth(lblRoomBed.Caption) +
      lblRoomBed.Margins.Left + lblRoomBed.Margins.Right + 4;

//  gpDemog.Width := controlled by right anchor

  iHeight := getMainFormTextHeight + 6;
  for i := 0 to gpDemog.RowCollection.Count - 1 do
    gpDemog.RowCollection[i].Value := iHeight;

  self.Height := gpDemog.RowCollection.Count + (iHeight + 1);
end;

procedure TfrmPtSelDemog.FormResize(Sender: TObject);
begin
  inherited;
  sbMain.VertScrollBar.Position := 0;
  Application.ProcessMessages;
end;

procedure TfrmPtSelDemog.acCVShowExecute(Sender: TObject);
begin
  inherited;
  acCVShow.Checked := not acCVShow.Checked;
end;

initialization
  SpecifyFormIsNotADialog(TfrmPtSelDemog);
end.
