unit ORRedirect;

interface

uses Winapi.Windows, System.SysUtils, System.RTTI, Vcl.Controls, Vcl.Forms;

const
  ASMCNT = 4;

type
  TASMData = array [0 .. ASMCNT] of byte;

  TRedirectToken = record
    Addr: ^TASMData;
    OldData, NewData: TASMData;
  end;

function BeginAPIRedirect(FromAPI, ToAPI: Pointer): TRedirectToken;
procedure EndAPIRedirect(Token: TRedirectToken);

type
  TSetFocusRedirect = class
  private
    class var FInitialized :Boolean;
    class var FControlSetFocusToken :TRedirectToken;
    class var FFormSetFocusToken :TRedirectToken;
    class var FFormSetActiveControlToken :TRedirectToken;
    class function AllowFocus(Control: TObject): boolean; static;
    class procedure NewControlSetFocus(Self: TObject); static;
    class procedure NewFormSetFocus(Self: TObject); static;
    class procedure NewSetActiveControl(Self: TObject; Control: TWinControl); static;
  public
    class procedure Init;
  end;

implementation

uses
  VAUtils;

function BeginAPIRedirect(FromAPI, ToAPI: Pointer): TRedirectToken;
type
  TPtrRec = record
    case integer of
      1:
        (lw: Pointer);
      2:
        (b1, b2, b3, b4: byte);
  end;

var
  rec: TPtrRec;
  dwNull: DWORD;

begin
  VirtualProtect(FromAPI, ASMCNT + 1, PAGE_EXECUTE_READWRITE, dwNull);
  with Result do
  begin
    Addr := FromAPI;
    OldData := Addr^;
    rec.lw := Pointer(integer(ToAPI) - integer(FromAPI) - 5);
    NewData[0] := $E9;
    NewData[1] := rec.b1;
    NewData[2] := rec.b2;
    NewData[3] := rec.b3;
    NewData[4] := rec.b4;
    Addr^ := NewData;
  end;
end;

procedure EndAPIRedirect(Token: TRedirectToken);
begin
  Token.Addr^ := Token.OldData;
end;

{ TSetFocusRedirect }

class function TSetFocusRedirect.AllowFocus(Control: TObject): boolean;
begin
  Result := (Control is TWinControl) and ShouldFocus(TWinControl(Control));
end;

class procedure TSetFocusRedirect.NewControlSetFocus(Self: TObject);
begin
  if AllowFocus(Self) then
    with FControlSetFocusToken do
    begin
      Addr^ := OldData;
      try
        TWinControl(Self).SetFocus;
      finally
        Addr^ := NewData;
      end;
    end;
end;

class procedure TSetFocusRedirect.NewFormSetFocus(Self: TObject);
begin
  if AllowFocus(Self) then
    with FFormSetFocusToken do
    begin
      Addr^ := OldData;
      try
        TWinControl(Self).SetFocus;
      finally
        Addr^ := NewData;
      end;
    end;
end;

class procedure TSetFocusRedirect.NewSetActiveControl(Self: TObject;
  Control: TWinControl);
begin
  if (Self is TCustomForm) and ((Control = nil) or AllowFocus(Control)) then
    with FFormSetActiveControlToken do
    begin
      Addr^ := OldData;
      try
        TCustomForm(Self).ActiveControl := Control;
      finally
        Addr^ := NewData;
      end;
    end;
end;

class procedure TSetFocusRedirect.Init;
var
  SetActiveControlAddr: Pointer;

  function FindSetActiveControlAddr: Pointer;
  var
    ctx: TRTTIContext;
    typ: TRttiType;
    prop: TRttiProperty;

  begin
    Result := nil;
    typ := ctx.GetType(TCustomForm);
    if assigned(typ) then
    begin
      prop := typ.GetProperty('ActiveControl');
      if assigned(prop) and (prop is TRttiInstanceProperty) then
        Result := TRttiInstanceProperty(prop).PropInfo^.SetProc;
    end;
  end;

begin
  if not FInitialized then
  begin
    SetActiveControlAddr := FindSetActiveControlAddr;
    if SetActiveControlAddr <> nil then
    begin
      FControlSetFocusToken := BeginAPIRedirect(@TWinControl.SetFocus,
        @NewControlSetFocus);
      FFormSetFocusToken := BeginAPIRedirect(@TCustomForm.SetFocus,
        @NewFormSetFocus);
      FFormSetActiveControlToken := BeginAPIRedirect(SetActiveControlAddr,
        @NewSetActiveControl);
    end;
    FInitialized := True;
  end;
end;

end.
