unit uTIU;

interface

uses SysUtils, Classes, ORNet, ORFn, rCore, uCore, uConst, ORCtrls, ComCtrls, Controls;

type

  TEditNoteRec = record
{$IFDEF GROUPNOTES}
    NoteIEN: int64;
    SignStatus: Integer; // Sign status;
{$ENDIF}
    DocType: Integer;
    IsNewNote: boolean;
    Title: Integer;
    TitleName: string;
    DateTime: TFMDateTime;
    Author: Int64;
    AuthorName: string;
    Cosigner: Int64;
    CosignerName: string;
    Subject: string;
    Location: Integer;
    LocationName: string;
    VisitDate: TFMDateTime;
    PkgRef: string;      // 'IEN;GMR(123,' or 'IEN;SRF('
    PkgIEN: integer;     // file IEN
    PkgPtr: string;      // 'GMR(123,' or 'SRF('
    NeedCPT: Boolean;
    Addend: Integer;
    LastCosigner: Int64;
    LastCosignerName: string;
    IDParent: integer;
    ClinProcSummCode: integer;
    ClinProcDateTime: TFMDateTime;
    Lines: TStrings;
    PRF_IEN: integer;
    ActionIEN: string;
{$IFDEF GROUPNOTES}
    GenLines: TStrings;  //general note content for all patients
    PtLines: TStrings;   //patient specific additional note's content
    ErrTxt: string;
{$ENDIF}
  end;

  TNoteRec = TEditNoteRec;

  TActionRec = record
    Success: Boolean;
    Reason: string;
  end;

  TCreatedDoc = record
    IEN: Integer;
    ErrorText: string;
  end;

  TTIUContext = record
    Changed: Boolean;
    BeginDate: string;
    EndDate: string;
    FMBeginDate: TFMDateTime;
    FMEndDate: TFMDateTime;
    Status: string;
    Author: int64;
    MaxDocs: integer;
    ShowSubject: Boolean;
    SortBy: string;
    ListAscending: Boolean;
    GroupBy: string;
    TreeAscending: Boolean;
    SearchField: string;
    KeyWord: string;
    Filtered: Boolean;
    SearchString: String;  // Text Search CQ: HDS00002856
  end ;

  TNoteTitles = class (TObject)
  public
    DfltTitle: Integer;
    DfltTitleName: string;
    ShortList: TStringList;
    constructor Create;
    destructor Destroy; override;
  end;

  TTIUPrefs = class (TObject)
  public
    DfltLoc: Integer;
    DfltLocName: string;
    SortAscending: Boolean;
    SortBy: string;    // D,R,S,A
    AskNoteSubject: Boolean;
    AskCosigner: Boolean;
    DfltCosigner: Int64;
    DfltCosignerName: string;
    MaxNotes: Integer;
  end;

//  notes tab specific procedures
//function MakeNoteDisplayText(RawText: string): string;
//function MakeConsultDisplayText(RawText:string): string;
//function SetLinesTo74ForSave(AStrings: TStrings; AParent: TWinControl): TStrings;

function EditNoteToStringList(aNote:TEditNoteRec): TStringList;
{
const
  TX_SAVE_ERROR1 = 'An error was encountered while trying to save the note you are editing.' + CRLF + CRLF + '    Error:  ';
  TX_SAVE_ERROR2 = CRLF + CRLF + 'Please try again now using CTRL-SHIFT-S, or ''Save without signature''.' + CRLF +
                   'These actions will improve the likelihood that no text will be lost.' + CRLF + CRLF +
                   'If problems continue, or network connectivity is lost, please copy all of your text to' + CRLF +
                   'the clipboard and paste it into Microsoft Word before continuing or before closing CPRS.';
  TC_SAVE_ERROR =  'Error saving note text';
}
implementation

(*
function MakeConsultDisplayText(RawText:string): string;
var
 x: string;
begin
   x := RawText;
   Result := Piece(x, U, 2) + ',' + Piece(x, U, 3) + ',' +
             Piece(x, U, 4) + ', '+ Piece(x, U, 5);
end;

function MakeNoteDisplayText(RawText: string): string;
var
  x: string;
begin
  x := RawText;
  if Piece(x, U, 1) = '' then
    Result := FormatFMDateTime('mmm dd,yy', MakeFMDateTime(Piece(x, U, 3))) + '  ' +
        Piece(x, U, 2) + ', ' + Piece(x, U, 6) + ', ' + Piece(Piece(x, U, 5), ';', 2)
  else if CharInSet(Piece(x, U, 1)[1], ['A', 'N', 'E']) then
    Result := Piece(x, U, 2)
  else
    Result := FormatFMDateTime('mmm dd,yy', MakeFMDateTime(Piece(x, U, 3))) + '  ' +
              Piece(x, U, 2) + ', ' + Piece(x, U, 6) + ', ' + Piece(Piece(x, U, 5), ';', 2);
end;

function SetLinesTo74ForSave(AStrings: TStrings; AParent: TWinControl): TStrings;
var
  ARichEdit74: TRichEdit;
begin
  Result := AStrings;
  ARichEdit74 := TRichEdit.Create(AParent);
  try
    ARichEdit74.Parent := AParent;
    ARichEdit74.Lines.Text := AStrings.Text;
    ARichEdit74.Width := 525;
    Result.Text := ARichEdit74.Lines.Text;
    //QuickCopy(ARichEdit74, Result);
  finally
    ARichEdit74.Free;
  end;
end;
*)

{ Progress Note Titles  -------------------------------------------------------------------- }
constructor TNoteTitles.Create;
{ creates an object to store progress note titles so only obtained from server once }
begin
  inherited Create;
  ShortList := TStringList.Create;
end;

destructor TNoteTitles.Destroy;
{ frees the lists that were used to store the progress note titles }
begin
  ShortList.Free;
  inherited Destroy;
end;

////////////////////////////////////////////////////////////////////////////////

function EditNoteToStringList(aNote:TEditNoteRec): TStringList;

  procedure AddStrings(aName:String;aText:TStrings);
  var
    s:String;
  begin
    Result.Add(aName);
    if assigned(aText) then
      for s in aText do
        Result.Add('    '+s);
  end;

begin
  Result := TSTringList.Create;
  with aNote do
  begin
    Result.Add('NoteIEN: '+IntToStr(NoteIEN));
    Result.Add('SignStatus: '+IntToStr(SignStatus));
    Result.Add('DocType: '+IntToStr(DocType));
    Result.Add('IsNewNote: '+BoolToStr(IsNewNote));
    Result.Add('Title: '+IntToStr(Title));
    Result.Add('TitleName: '+TitleName);
    Result.Add('DateTime: '+FloatToStr(Datetime));
    Result.Add('Author: '+IntToStr(Author));
    Result.Add('AuthorName: '+AuthorName);
    Result.Add('Cosigner: '+IntToStr(Cosigner));
    Result.Add('CosignerName: '+CosignerName);
    Result.Add('Subject: '+Subject);
    Result.Add('Location: '+IntToStr(Location));
    Result.Add('LocationName: '+LocationName);
    Result.Add('VisitDate: '+FloatToStr(VisitDate));
    Result.Add('PkgRef: '+PkgRef);
    Result.Add('PkgIEN: '+IntToStr(PkgIEN));
    Result.Add('PkgPtr: '+PkgPtr);
    Result.Add('NeedCPT: '+BoolToStr(NeedCPT));
    Result.Add('Addend: '+IntToStr(Addend));
    Result.Add('LstCosigner: '+IntToStr(LastCosigner));
    Result.Add('LastCosignerName: '+LastCosignerName);
    Result.Add('IDParent: '+IntToStr(IDParent));
    Result.Add('ClinProcSummCode: '+IntToStr(ClinProcSummCode));
    Result.Add('ClinProcDateTime: '+FloatToStr(ClinProcDateTime));

    AddStrings('Lines',Lines);

    Result.Add('PRF_IEN: '+IntToStr(PRF_IEN));
    Result.Add('ActionIEN: '+ActionIEN);

    AddStrings('GenLines',GenLines);
    AddStrings('PtLines: ',PtLines);
    Result.Add('ErrTxt: '+ErrTxt);

  end;
end;

end.
